@@ -25,6 +25,7 @@ import webob.dec
 import webob.exc
 
 from nova.api.openstack import wsgi
+from nova import notifications
 from nova.openstack.common import log as logging
 from nova import utils
 from nova import wsgi as base_wsgi
@@ -70,6 +71,8 @@ class FaultWrapper(base_wsgi.Middleware):
         if safe:
             outer.explanation = '%s: %s' % (inner.__class__.__name__,
                                             unicode(inner))
+
+        notifications.send_api_fault(req.url, status, inner)
         return wsgi.Fault(outer)
 
     @webob.dec.wsgify(RequestClass=wsgi.Request)
@@ -99,7 +102,7 @@ class APIMapper(routes.Mapper):
 
 class ProjectMapper(APIMapper):
     def resource(self, member_name, collection_name, **kwargs):
-        if not ('parent_resource' in kwargs):
+        if 'parent_resource' not in kwargs:
             kwargs['path_prefix'] = '{project_id}/'
         else:
             parent_resource = kwargs['parent_resource']
@@ -121,10 +124,10 @@ class APIRouter(base_wsgi.Router):
 
     @classmethod
     def factory(cls, global_config, **local_config):
-        """Simple paste factory, :class:`nova.wsgi.Router` doesn't have one"""
+        """Simple paste factory, :class:`nova.wsgi.Router` doesn't have one."""
         return cls()
 
-    def __init__(self, ext_mgr=None):
+    def __init__(self, ext_mgr=None, init_only=None):
         if ext_mgr is None:
             if self.ExtensionManager:
                 ext_mgr = self.ExtensionManager()
@@ -133,16 +136,19 @@ class APIRouter(base_wsgi.Router):
 
         mapper = ProjectMapper()
         self.resources = {}
-        self._setup_routes(mapper, ext_mgr)
-        self._setup_ext_routes(mapper, ext_mgr)
+        self._setup_routes(mapper, ext_mgr, init_only)
+        self._setup_ext_routes(mapper, ext_mgr, init_only)
         self._setup_extensions(ext_mgr)
         super(APIRouter, self).__init__(mapper)
 
-    def _setup_ext_routes(self, mapper, ext_mgr):
+    def _setup_ext_routes(self, mapper, ext_mgr, init_only):
         for resource in ext_mgr.get_resources():
             LOG.debug(_('Extended resource: %s'),
                       resource.collection)
 
+            if init_only is not None and resource.collection not in init_only:
+                continue
+
             inherits = None
             if resource.inherits:
                 inherits = self.resources.get(resource.inherits)
@@ -183,5 +189,5 @@ class APIRouter(base_wsgi.Router):
             resource.register_actions(controller)
             resource.register_extensions(controller)
 
-    def _setup_routes(self, mapper, ext_mgr):
-        raise NotImplementedError
+    def _setup_routes(self, mapper, ext_mgr, init_only):
+        raise NotImplementedError()
