@@ -1,4 +1,4 @@
-#   Copyright 2011 OpenStack, LLC.
+#   Copyright 2011 OpenStack Foundation
 #
 #   Licensed under the Apache License, Version 2.0 (the "License"); you may
 #   not use this file except in compliance with the License. You may obtain
@@ -24,14 +24,10 @@ from nova.api.openstack import wsgi
 from nova import compute
 from nova.compute import vm_states
 from nova import exception
-from nova import flags
 from nova.openstack.common import log as logging
 
-
-FLAGS = flags.FLAGS
 LOG = logging.getLogger(__name__)
 
-
 # States usable in resetState action
 state_map = dict(active=vm_states.ACTIVE, error=vm_states.ERROR)
 
@@ -50,7 +46,7 @@ class AdminActionsController(wsgi.Controller):
 
     @wsgi.action('pause')
     def _pause(self, req, id, body):
-        """Permit Admins to pause the server"""
+        """Permit Admins to pause the server."""
         ctxt = req.environ['nova.context']
         authorize(ctxt, 'pause')
         try:
@@ -67,7 +63,7 @@ class AdminActionsController(wsgi.Controller):
 
     @wsgi.action('unpause')
     def _unpause(self, req, id, body):
-        """Permit Admins to unpause the server"""
+        """Permit Admins to unpause the server."""
         ctxt = req.environ['nova.context']
         authorize(ctxt, 'unpause')
         try:
@@ -84,7 +80,7 @@ class AdminActionsController(wsgi.Controller):
 
     @wsgi.action('suspend')
     def _suspend(self, req, id, body):
-        """Permit admins to suspend the server"""
+        """Permit admins to suspend the server."""
         context = req.environ['nova.context']
         authorize(context, 'suspend')
         try:
@@ -101,7 +97,7 @@ class AdminActionsController(wsgi.Controller):
 
     @wsgi.action('resume')
     def _resume(self, req, id, body):
-        """Permit admins to resume the server from suspend"""
+        """Permit admins to resume the server from suspend."""
         context = req.environ['nova.context']
         authorize(context, 'resume')
         try:
@@ -118,7 +114,7 @@ class AdminActionsController(wsgi.Controller):
 
     @wsgi.action('migrate')
     def _migrate(self, req, id, body):
-        """Permit admins to migrate a server to a new host"""
+        """Permit admins to migrate a server to a new host."""
         context = req.environ['nova.context']
         authorize(context, 'migrate')
         try:
@@ -134,7 +130,7 @@ class AdminActionsController(wsgi.Controller):
 
     @wsgi.action('resetNetwork')
     def _reset_network(self, req, id, body):
-        """Permit admins to reset networking on an server"""
+        """Permit admins to reset networking on a server."""
         context = req.environ['nova.context']
         authorize(context, 'resetNetwork')
         try:
@@ -148,7 +144,7 @@ class AdminActionsController(wsgi.Controller):
 
     @wsgi.action('injectNetworkInfo')
     def _inject_network_info(self, req, id, body):
-        """Permit admins to inject network info into a server"""
+        """Permit admins to inject network info into a server."""
         context = req.environ['nova.context']
         authorize(context, 'injectNetworkInfo')
         try:
@@ -164,7 +160,7 @@ class AdminActionsController(wsgi.Controller):
 
     @wsgi.action('lock')
     def _lock(self, req, id, body):
-        """Permit admins to lock a server"""
+        """Permit admins to lock a server."""
         context = req.environ['nova.context']
         authorize(context, 'lock')
         try:
@@ -180,7 +176,7 @@ class AdminActionsController(wsgi.Controller):
 
     @wsgi.action('unlock')
     def _unlock(self, req, id, body):
-        """Permit admins to lock a server"""
+        """Permit admins to lock a server."""
         context = req.environ['nova.context']
         authorize(context, 'unlock')
         try:
@@ -270,7 +266,7 @@ class AdminActionsController(wsgi.Controller):
 
     @wsgi.action('os-migrateLive')
     def _migrate_live(self, req, id, body):
-        """Permit admins to (live) migrate a server to a new host"""
+        """Permit admins to (live) migrate a server to a new host."""
         context = req.environ["nova.context"]
         authorize(context, 'migrateLive')
 
@@ -286,11 +282,18 @@ class AdminActionsController(wsgi.Controller):
             instance = self.compute_api.get(context, id)
             self.compute_api.live_migrate(context, instance, block_migration,
                                           disk_over_commit, host)
-        except exception.ComputeServiceUnavailable as ex:
+        except (exception.ComputeServiceUnavailable,
+                exception.InvalidHypervisorType,
+                exception.UnableToMigrateToSelf,
+                exception.DestinationHypervisorTooOld) as ex:
             raise exc.HTTPBadRequest(explanation=ex.format_message())
         except Exception:
-            msg = _("Live migration of instance %(id)s to host %(host)s"
-                    " failed") % locals()
+            if host is None:
+                msg = _("Live migration of instance %(id)s to another host"
+                        " failed") % locals()
+            else:
+                msg = _("Live migration of instance %(id)s to host %(host)s"
+                        " failed") % locals()
             LOG.exception(msg)
             # Return messages from scheduler
             raise exc.HTTPBadRequest(explanation=msg)
@@ -313,9 +316,7 @@ class AdminActionsController(wsgi.Controller):
 
         try:
             instance = self.compute_api.get(context, id)
-            self.compute_api.update(context, instance,
-                                    vm_state=state,
-                                    task_state=None)
+            self.compute_api.update_state(context, instance, state)
         except exception.InstanceNotFound:
             raise exc.HTTPNotFound(_("Server not found"))
         except Exception:
