@@ -43,7 +43,7 @@ class FlavorManageController(wsgi.Controller):
 
         try:
             flavor = instance_types.get_instance_type_by_flavor_id(
-                    id, read_deleted="no")
+                    id, ctxt=context, read_deleted="no")
         except exception.NotFound, e:
             raise webob.exc.HTTPNotFound(explanation=e.format_message())
 
@@ -73,7 +73,8 @@ class FlavorManageController(wsgi.Controller):
                                            root_gb, ephemeral_gb, flavorid,
                                            swap, rxtx_factor, is_public)
             req.cache_db_flavor(flavor)
-        except exception.InstanceTypeExists as err:
+        except (exception.InstanceTypeExists,
+                exception.InstanceTypeIdExists) as err:
             raise webob.exc.HTTPConflict(explanation=err.format_message())
 
         return self._view_builder.show(req, flavor)
