@@ -18,11 +18,11 @@
 Client side of the consoleauth RPC API.
 """
 
-from nova import flags
+from oslo.config import cfg
+
 import nova.openstack.common.rpc.proxy
 
-
-FLAGS = flags.FLAGS
+CONF = cfg.CONF
 
 
 class ConsoleAuthAPI(nova.openstack.common.rpc.proxy.RpcProxy):
@@ -31,6 +31,9 @@ class ConsoleAuthAPI(nova.openstack.common.rpc.proxy.RpcProxy):
     API version history:
 
         1.0 - Initial version.
+        1.1 - Added get_backdoor_port()
+        1.2 - Added instance_uuid to authorize_console, and
+              delete_tokens_for_instance
     '''
 
     #
@@ -45,7 +48,7 @@ class ConsoleAuthAPI(nova.openstack.common.rpc.proxy.RpcProxy):
 
     def __init__(self):
         super(ConsoleAuthAPI, self).__init__(
-                topic=FLAGS.consoleauth_topic,
+                topic=CONF.consoleauth_topic,
                 default_version=self.BASE_RPC_API_VERSION)
 
     def authorize_console(self, ctxt, token, console_type, host, port,
@@ -57,12 +60,18 @@ class ConsoleAuthAPI(nova.openstack.common.rpc.proxy.RpcProxy):
                               token=token, console_type=console_type,
                               host=host, port=port,
                               internal_access_path=internal_access_path,
-                              instance_uuid=instance_uuid))
+                              instance_uuid=instance_uuid),
+                version="1.2")
 
     def check_token(self, ctxt, token):
         return self.call(ctxt, self.make_msg('check_token', token=token))
 
     def delete_tokens_for_instance(self, ctxt, instance_uuid):
-        return self.call(ctxt,
+        return self.cast(ctxt,
                 self.make_msg('delete_tokens_for_instance',
-                              instance_uuid=instance_uuid))
+                              instance_uuid=instance_uuid),
+                version="1.2")
+
+    def get_backdoor_port(self, ctxt, host):
+        return self.call(ctxt, self.make_msg('get_backdoor_port'),
+                         version='1.1')
