@@ -24,19 +24,18 @@ import os
 import shutil
 import tarfile
 import tempfile
-from xml.etree import ElementTree
 
 import boto.s3.connection
 import eventlet
+from lxml import etree
+from oslo.config import cfg
 
-from nova import rpc
-from nova import exception
-from nova import flags
-from nova import image
-from nova import log as logging
-from nova.openstack.common import cfg
-from nova import utils
 from nova.api.ec2 import ec2utils
+import nova.cert.rpcapi
+from nova import exception
+from nova.image import glance
+from nova.openstack.common import log as logging
+from nova import utils
 
 
 LOG = logging.getLogger(__name__)
@@ -45,6 +44,13 @@ s3_opts = [
     cfg.StrOpt('image_decryption_dir',
                default='/tmp',
                help='parent dir for tempdir used for image decryption'),
+    cfg.StrOpt('s3_host',
+               default='$my_ip',
+               help='hostname or ip for openstack to use when accessing '
+                    'the s3 api'),
+    cfg.IntOpt('s3_port',
+               default=3333,
+               help='port used when accessing the s3 api'),
     cfg.StrOpt('s3_access_key',
                default='notchecked',
                help='access key to use for s3 server for images'),
@@ -60,15 +66,17 @@ s3_opts = [
                     'when downloading from s3'),
     ]
 
-FLAGS = flags.FLAGS
-FLAGS.register_opts(s3_opts)
+CONF = cfg.CONF
+CONF.register_opts(s3_opts)
+CONF.import_opt('my_ip', 'nova.netconf')
 
 
 class S3ImageService(object):
     """Wraps an existing image service to support s3 based register."""
 
     def __init__(self, service=None, *args, **kwargs):
-        self.service = service or image.get_default_image_service()
+        self.cert_rpcapi = nova.cert.rpcapi.CertAPI()
+        self.service = service or glance.get_default_image_service()
         self.service.__init__(*args, **kwargs)
 
     def _translate_uuids_to_ids(self, context, images):
@@ -135,16 +143,11 @@ class S3ImageService(object):
         image = self.service.update(context, image_uuid, metadata, data)
         return self._translate_uuid_to_id(context, image)
 
-    def index(self, context):
+    def detail(self, context, **kwargs):
         #NOTE(bcwaldon): sort asc to make sure we assign lower ids
         # to older images
-        images = self.service.index(context, sort_dir='asc')
-        return self._translate_uuids_to_ids(context, images)
-
-    def detail(self, context):
-        #NOTE(bcwaldon): sort asc to make sure we assign lower ids
-        # to older images
-        images = self.service.detail(context, sort_dir='asc')
+        kwargs.setdefault('sort_dir', 'asc')
+        images = self.service.detail(context, **kwargs)
         return self._translate_uuids_to_ids(context, images)
 
     def show(self, context, image_id):
@@ -152,25 +155,21 @@ class S3ImageService(object):
         image = self.service.show(context, image_uuid)
         return self._translate_uuid_to_id(context, image)
 
-    def show_by_name(self, context, name):
-        image = self.service.show_by_name(context, name)
-        return self._translate_uuid_to_id(context, image)
-
     @staticmethod
     def _conn(context):
         # NOTE(vish): access and secret keys for s3 server are not
         #             checked in nova-objectstore
-        access = FLAGS.s3_access_key
-        if FLAGS.s3_affix_tenant:
+        access = CONF.s3_access_key
+        if CONF.s3_affix_tenant:
             access = '%s:%s' % (access, context.project_id)
-        secret = FLAGS.s3_secret_key
+        secret = CONF.s3_secret_key
         calling = boto.s3.connection.OrdinaryCallingFormat()
         return boto.s3.connection.S3Connection(aws_access_key_id=access,
                                                aws_secret_access_key=secret,
-                                               is_secure=FLAGS.s3_use_ssl,
+                                               is_secure=CONF.s3_use_ssl,
                                                calling_format=calling,
-                                               port=FLAGS.s3_port,
-                                               host=FLAGS.s3_host)
+                                               port=CONF.s3_port,
+                                               host=CONF.s3_host)
 
     @staticmethod
     def _download_file(bucket, filename, local_dir):
@@ -180,7 +179,7 @@ class S3ImageService(object):
         return local_filename
 
     def _s3_parse_manifest(self, context, metadata, manifest):
-        manifest = ElementTree.fromstring(manifest)
+        manifest = etree.fromstring(manifest)
         image_format = 'ami'
         image_type = 'machine'
 
@@ -232,9 +231,6 @@ class S3ImageService(object):
         properties = metadata['properties']
         properties['architecture'] = arch
 
-        if FLAGS.auth_strategy == 'deprecated':
-            properties['project_id'] = context.project_id
-
         def _translate_dependent_image_id(image_key, image_id):
             image_uuid = ec2utils.ec2_id_to_glance_id(context, image_id)
             properties[image_key] = image_uuid
@@ -271,7 +267,7 @@ class S3ImageService(object):
     def _s3_create(self, context, metadata):
         """Gets a manifest from s3 and makes an image."""
 
-        image_path = tempfile.mkdtemp(dir=FLAGS.image_decryption_dir)
+        image_path = tempfile.mkdtemp(dir=CONF.image_decryption_dir)
 
         image_location = metadata['properties']['image_location']
         bucket_name = image_location.split('/')[0]
@@ -289,8 +285,18 @@ class S3ImageService(object):
             context.update_store()
             log_vars = {'image_location': image_location,
                         'image_path': image_path}
-            metadata['properties']['image_state'] = 'downloading'
-            self.service.update(context, image_uuid, metadata)
+
+            def _update_image_state(context, image_uuid, image_state):
+                metadata = {'properties': {'image_state': image_state}}
+                self.service.update(context, image_uuid, metadata,
+                                    purge_props=False)
+
+            def _update_image_data(context, image_uuid, image_data):
+                metadata = {}
+                self.service.update(context, image_uuid, metadata, image_data,
+                                    purge_props=False)
+
+            _update_image_state(context, image_uuid, 'downloading')
 
             try:
                 parts = []
@@ -311,12 +317,10 @@ class S3ImageService(object):
             except Exception:
                 LOG.exception(_("Failed to download %(image_location)s "
                                 "to %(image_path)s"), log_vars)
-                metadata['properties']['image_state'] = 'failed_download'
-                self.service.update(context, image_uuid, metadata)
+                _update_image_state(context, image_uuid, 'failed_download')
                 return
 
-            metadata['properties']['image_state'] = 'decrypting'
-            self.service.update(context, image_uuid, metadata)
+            _update_image_state(context, image_uuid, 'decrypting')
 
             try:
                 hex_key = manifest.find('image/ec2_encrypted_key').text
@@ -330,38 +334,33 @@ class S3ImageService(object):
             except Exception:
                 LOG.exception(_("Failed to decrypt %(image_location)s "
                                 "to %(image_path)s"), log_vars)
-                metadata['properties']['image_state'] = 'failed_decrypt'
-                self.service.update(context, image_uuid, metadata)
+                _update_image_state(context, image_uuid, 'failed_decrypt')
                 return
 
-            metadata['properties']['image_state'] = 'untarring'
-            self.service.update(context, image_uuid, metadata)
+            _update_image_state(context, image_uuid, 'untarring')
 
             try:
                 unz_filename = self._untarzip_image(image_path, dec_filename)
             except Exception:
                 LOG.exception(_("Failed to untar %(image_location)s "
                                 "to %(image_path)s"), log_vars)
-                metadata['properties']['image_state'] = 'failed_untar'
-                self.service.update(context, image_uuid, metadata)
+                _update_image_state(context, image_uuid, 'failed_untar')
                 return
 
-            metadata['properties']['image_state'] = 'uploading'
-            self.service.update(context, image_uuid, metadata)
+            _update_image_state(context, image_uuid, 'uploading')
             try:
                 with open(unz_filename) as image_file:
-                    self.service.update(context, image_uuid,
-                                        metadata, image_file)
+                    _update_image_data(context, image_uuid, image_file)
             except Exception:
                 LOG.exception(_("Failed to upload %(image_location)s "
                                 "to %(image_path)s"), log_vars)
-                metadata['properties']['image_state'] = 'failed_upload'
-                self.service.update(context, image_uuid, metadata)
+                _update_image_state(context, image_uuid, 'failed_upload')
                 return
 
-            metadata['properties']['image_state'] = 'available'
-            metadata['status'] = 'active'
-            self.service.update(context, image_uuid, metadata)
+            metadata = {'status': 'active',
+                        'properties': {'image_state': 'available'}}
+            self.service.update(context, image_uuid, metadata,
+                    purge_props=False)
 
             shutil.rmtree(image_path)
 
@@ -369,25 +368,22 @@ class S3ImageService(object):
 
         return image
 
-    @staticmethod
-    def _decrypt_image(context, encrypted_filename, encrypted_key,
+    def _decrypt_image(self, context, encrypted_filename, encrypted_key,
                        encrypted_iv, decrypted_filename):
         elevated = context.elevated()
         try:
-            key = rpc.call(elevated, FLAGS.cert_topic,
-                           {"method": "decrypt_text",
-                            "args": {"project_id": context.project_id,
-                                     "text": base64.b64encode(encrypted_key)}})
+            key = self.cert_rpcapi.decrypt_text(elevated,
+                    project_id=context.project_id,
+                    text=base64.b64encode(encrypted_key))
         except Exception, exc:
-            raise exception.Error(_('Failed to decrypt private key: %s')
-                                  % exc)
+            msg = _('Failed to decrypt private key: %s') % exc
+            raise exception.NovaException(msg)
         try:
-            iv = rpc.call(elevated, FLAGS.cert_topic,
-                          {"method": "decrypt_text",
-                           "args": {"project_id": context.project_id,
-                                    "text": base64.b64encode(encrypted_iv)}})
+            iv = self.cert_rpcapi.decrypt_text(elevated,
+                    project_id=context.project_id,
+                    text=base64.b64encode(encrypted_iv))
         except Exception, exc:
-            raise exception.Error(_('Failed to decrypt initialization '
+            raise exception.NovaException(_('Failed to decrypt initialization '
                                     'vector: %s') % exc)
 
         try:
@@ -398,19 +394,19 @@ class S3ImageService(object):
                           '-iv', '%s' % (iv,),
                           '-out', '%s' % (decrypted_filename,))
         except exception.ProcessExecutionError, exc:
-            raise exception.Error(_('Failed to decrypt image file '
+            raise exception.NovaException(_('Failed to decrypt image file '
                                     '%(image_file)s: %(err)s') %
                                     {'image_file': encrypted_filename,
                                      'err': exc.stdout})
 
     @staticmethod
     def _test_for_malicious_tarball(path, filename):
-        """Raises exception if extracting tarball would escape extract path"""
+        """Raises exception if extracting tarball would escape extract path."""
         tar_file = tarfile.open(filename, 'r|gz')
         for n in tar_file.getnames():
             if not os.path.abspath(os.path.join(path, n)).startswith(path):
                 tar_file.close()
-                raise exception.Error(_('Unsafe filenames in image'))
+                raise exception.NovaException(_('Unsafe filenames in image'))
         tar_file.close()
 
     @staticmethod
