@@ -459,34 +459,34 @@ class CinderCloudTestCase(test.TestCase):
 
     _expected_block_device_mapping0 = [
         {'deviceName': '/dev/sdb1',
-         'ebs': {'status': 'in-use',
+         'ebs': {'status': 'attached',
                  'deleteOnTermination': False,
-                 'volumeId': '2',
+                 'volumeId': 'vol-00000002',
                  }},
         {'deviceName': '/dev/sdb2',
-         'ebs': {'status': 'in-use',
+         'ebs': {'status': 'attached',
                  'deleteOnTermination': False,
-                 'volumeId': '3',
+                 'volumeId': 'vol-00000003',
                  }},
         {'deviceName': '/dev/sdb3',
-         'ebs': {'status': 'in-use',
+         'ebs': {'status': 'attached',
                  'deleteOnTermination': True,
-                 'volumeId': '5',
+                 'volumeId': 'vol-00000005',
                  }},
         {'deviceName': '/dev/sdb4',
-         'ebs': {'status': 'in-use',
+         'ebs': {'status': 'attached',
                  'deleteOnTermination': False,
-                 'volumeId': '7',
+                 'volumeId': 'vol-00000007',
                  }},
         {'deviceName': '/dev/sdb5',
-         'ebs': {'status': 'in-use',
+         'ebs': {'status': 'attached',
                  'deleteOnTermination': False,
-                 'volumeId': '9',
+                 'volumeId': 'vol-00000009',
                  }},
         {'deviceName': '/dev/sdb6',
-         'ebs': {'status': 'in-use',
+         'ebs': {'status': 'attached',
                  'deleteOnTermination': False,
-                 'volumeId': '11', }}]
+                 'volumeId': 'vol-0000000b', }}]
         # NOTE(yamahata): swap/ephemeral device case isn't supported yet.
 
     _expected_instance_bdm2 = {
