@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -25,13 +25,12 @@ from nova.tests.api.openstack import fakes
 
 
 def quota_set(id):
-    return {'quota_set': {'id': id, 'metadata_items': 128, 'volumes': 10,
-                          'gigabytes': 1000, 'ram': 51200, 'floating_ips': 10,
-                          'fixed_ips': -1, 'instances': 10,
-                          'injected_files': 5, 'cores': 20,
-                          'injected_file_content_bytes': 10240,
-                          'security_groups': 10, 'security_group_rules': 20,
-                          'key_pairs': 100, 'injected_file_path_bytes': 255}}
+    return {'quota_set': {'id': id, 'metadata_items': 128,
+            'ram': 51200, 'floating_ips': 10, 'fixed_ips': -1,
+            'instances': 10, 'injected_files': 5, 'cores': 20,
+            'injected_file_content_bytes': 10240,
+            'security_groups': 10, 'security_group_rules': 20,
+            'key_pairs': 100, 'injected_file_path_bytes': 255}}
 
 
 class QuotaSetsTest(test.TestCase):
@@ -45,18 +44,15 @@ class QuotaSetsTest(test.TestCase):
             'instances': 10,
             'cores': 20,
             'ram': 51200,
-            'volumes': 10,
             'floating_ips': 10,
             'fixed_ips': -1,
             'metadata_items': 128,
-            'gigabytes': 1000,
             'injected_files': 5,
             'injected_file_path_bytes': 255,
             'injected_file_content_bytes': 10240,
             'security_groups': 10,
             'security_group_rules': 20,
-            'key_pairs': 100,
-            }
+            'key_pairs': 100}
 
         quota_set = self.controller._format_quota_set('1234', raw_quota_set)
         qs = quota_set['quota_set']
@@ -65,8 +61,6 @@ class QuotaSetsTest(test.TestCase):
         self.assertEqual(qs['instances'], 10)
         self.assertEqual(qs['cores'], 20)
         self.assertEqual(qs['ram'], 51200)
-        self.assertEqual(qs['volumes'], 10)
-        self.assertEqual(qs['gigabytes'], 1000)
         self.assertEqual(qs['floating_ips'], 10)
         self.assertEqual(qs['fixed_ips'], -1)
         self.assertEqual(qs['metadata_items'], 128)
@@ -88,8 +82,6 @@ class QuotaSetsTest(test.TestCase):
                     'instances': 10,
                     'cores': 20,
                     'ram': 51200,
-                    'volumes': 10,
-                    'gigabytes': 1000,
                     'floating_ips': 10,
                     'fixed_ips': -1,
                     'metadata_items': 128,
@@ -98,8 +90,7 @@ class QuotaSetsTest(test.TestCase):
                     'injected_file_content_bytes': 10240,
                     'security_groups': 10,
                     'security_group_rules': 20,
-                    'key_pairs': 100,
-                    }}
+                    'key_pairs': 100}}
 
         self.assertEqual(res_dict, expected)
 
@@ -117,10 +108,9 @@ class QuotaSetsTest(test.TestCase):
 
     def test_quotas_update_as_admin(self):
         body = {'quota_set': {'instances': 50, 'cores': 50,
-                              'ram': 51200, 'volumes': 10,
-                              'gigabytes': 1000, 'floating_ips': 10,
-                              'metadata_items': 128, 'injected_files': 5,
-                              'fixed_ips': -1,
+                              'ram': 51200, 'floating_ips': 10,
+                              'fixed_ips': -1, 'metadata_items': 128,
+                              'injected_files': 5,
                               'injected_file_content_bytes': 10240,
                               'injected_file_path_bytes': 255,
                               'security_groups': 10,
@@ -135,10 +125,9 @@ class QuotaSetsTest(test.TestCase):
 
     def test_quotas_update_as_user(self):
         body = {'quota_set': {'instances': 50, 'cores': 50,
-                              'ram': 51200, 'volumes': 10,
-                              'gigabytes': 1000, 'floating_ips': 10,
-                              'metadata_items': 128, 'injected_files': 5,
-                              'fixed_ips': -1,
+                              'ram': 51200, 'floating_ips': 10,
+                              'fixed_ips': -1, 'metadata_items': 128,
+                              'injected_files': 5,
                               'injected_file_content_bytes': 10240,
                               'security_groups': 10,
                               'security_group_rules': 20,
@@ -148,10 +137,9 @@ class QuotaSetsTest(test.TestCase):
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.update,
                           req, 'update_me', body)
 
-    def test_quotas_update_invalid_limit(self):
-        body = {'quota_set': {'instances': -2, 'cores': -2,
-                              'ram': -2, 'volumes': -2,
-                              'gigabytes': -2, 'floating_ips': -2,
+    def test_quotas_update_invalid_key(self):
+        body = {'quota_set': {'instances2': -2, 'cores': -2,
+                              'ram': -2, 'floating_ips': -2,
                               'metadata_items': -2, 'injected_files': -2,
                               'injected_file_content_bytes': -2}}
 
@@ -160,6 +148,59 @@ class QuotaSetsTest(test.TestCase):
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.update,
                           req, 'update_me', body)
 
+    def test_quotas_update_invalid_limit(self):
+        body = {'quota_set': {'instances': -2, 'cores': -2,
+                              'ram': -2, 'floating_ips': -2, 'fixed_ips': -2,
+                              'metadata_items': -2, 'injected_files': -2,
+                              'injected_file_content_bytes': -2}}
+
+        req = fakes.HTTPRequest.blank('/v2/fake4/os-quota-sets/update_me',
+                                      use_admin_context=True)
+        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.update,
+                          req, 'update_me', body)
+
+    def test_quotas_update_invalid_value(self):
+        expected_resp = {'quota_set': {
+                              'instances': 50, 'cores': 50,
+                              'ram': 51200, 'floating_ips': 10,
+                              'fixed_ips': -1, 'metadata_items': 128,
+                              'injected_files': 5,
+                              'injected_file_content_bytes': 10240,
+                              'injected_file_path_bytes': 255,
+                              'security_groups': 10,
+                              'security_group_rules': 20,
+                              'key_pairs': 100}}
+
+        # when PUT JSON format with empty string for quota
+        body = {'quota_set': {'instances': 50, 'cores': 50,
+                              'ram': '', 'floating_ips': 10,
+                              'fixed_ips': -1, 'metadata_items': 128,
+                              'injected_files': 5,
+                              'injected_file_content_bytes': 10240,
+                              'injected_file_path_bytes': 255,
+                              'security_groups': 10,
+                              'security_group_rules': 20,
+                              'key_pairs': 100}}
+        req = fakes.HTTPRequest.blank('/v2/fake4/os-quota-sets/update_me',
+                                      use_admin_context=True)
+        res_dict = self.controller.update(req, 'update_me', body)
+        self.assertEqual(res_dict, expected_resp)
+
+        # when PUT XML format with empty string for quota
+        body = {'quota_set': {'instances': 50, 'cores': 50,
+                              'ram': {}, 'floating_ips': 10,
+                              'fixed_ips': -1, 'metadata_items': 128,
+                              'injected_files': 5,
+                              'injected_file_content_bytes': 10240,
+                              'injected_file_path_bytes': 255,
+                              'security_groups': 10,
+                              'security_group_rules': 20,
+                              'key_pairs': 100}}
+        req = fakes.HTTPRequest.blank('/v2/fake4/os-quota-sets/update_me',
+                                      use_admin_context=True)
+        res_dict = self.controller.update(req, 'update_me', body)
+        self.assertEqual(res_dict, expected_resp)
+
 
 class QuotaXMLSerializerTest(test.TestCase):
     def setUp(self):
@@ -173,8 +214,6 @@ class QuotaXMLSerializerTest(test.TestCase):
                 metadata_items=10,
                 injected_file_path_bytes=255,
                 injected_file_content_bytes=20,
-                volumes=30,
-                gigabytes=40,
                 ram=50,
                 floating_ips=60,
                 fixed_ips=-1,
@@ -186,7 +225,6 @@ class QuotaXMLSerializerTest(test.TestCase):
                 cores=90))
         text = self.serializer.serialize(exemplar)
 
-        print text
         tree = etree.fromstring(text)
 
         self.assertEqual('quota_set', tree.tag)
@@ -200,8 +238,6 @@ class QuotaXMLSerializerTest(test.TestCase):
         exemplar = dict(quota_set=dict(
                 metadata_items='10',
                 injected_file_content_bytes='20',
-                volumes='30',
-                gigabytes='40',
                 ram='50',
                 floating_ips='60',
                 fixed_ips='-1',
@@ -216,8 +252,6 @@ class QuotaXMLSerializerTest(test.TestCase):
                   '<metadata_items>10</metadata_items>'
                   '<injected_file_content_bytes>20'
                   '</injected_file_content_bytes>'
-                  '<volumes>30</volumes>'
-                  '<gigabytes>40</gigabytes>'
                   '<ram>50</ram>'
                   '<floating_ips>60</floating_ips>'
                   '<fixed_ips>-1</fixed_ips>'
