@@ -18,73 +18,11 @@ import webob
 
 from nova.api.openstack.compute.contrib import volumes
 from nova import context
-from nova import exception
-from nova import flags
 from nova.openstack.common import jsonutils
-from nova.openstack.common import log as logging
 from nova.openstack.common import timeutils
 from nova import test
 from nova.tests.api.openstack import fakes
-from nova import volume
-
-FLAGS = flags.FLAGS
-
-LOG = logging.getLogger(__name__)
-
-_last_param = {}
-
-
-def _get_default_snapshot_param():
-    return {
-        'id': 123,
-        'volume_id': 12,
-        'status': 'available',
-        'volume_size': 100,
-        'created_at': None,
-        'display_name': 'Default name',
-        'display_description': 'Default description',
-        }
-
-
-def stub_snapshot_create(self, context, volume_id, name, description):
-    global _last_param
-    snapshot = _get_default_snapshot_param()
-    snapshot['volume_id'] = volume_id
-    snapshot['display_name'] = name
-    snapshot['display_description'] = description
-
-    LOG.debug(_("_create: %s"), snapshot)
-    _last_param = snapshot
-    return snapshot
-
-
-def stub_snapshot_delete(self, context, snapshot):
-    global _last_param
-    _last_param = snapshot
-
-    LOG.debug(_("_delete: %s"), locals())
-    if snapshot['id'] != '123':
-        raise exception.NotFound
-
-
-def stub_snapshot_get(self, context, snapshot_id):
-    global _last_param
-    _last_param = dict(snapshot_id=snapshot_id)
-
-    LOG.debug(_("_get: %s"), locals())
-    if snapshot_id != '123':
-        raise exception.NotFound
-
-    param = _get_default_snapshot_param()
-    param['id'] = snapshot_id
-    return param
-
-
-def stub_snapshot_get_all(self, context):
-    LOG.debug(_("_get_all: %s"), locals())
-    param = _get_default_snapshot_param()
-    param['id'] = 123
-    return [param]
+from nova.volume import cinder
 
 
 class SnapshotApiTest(test.TestCase):
@@ -92,21 +30,25 @@ class SnapshotApiTest(test.TestCase):
         super(SnapshotApiTest, self).setUp()
         fakes.stub_out_networking(self.stubs)
         fakes.stub_out_rate_limiting(self.stubs)
-        self.stubs.Set(volume.api.API, "create_snapshot", stub_snapshot_create)
-        self.stubs.Set(volume.api.API, "create_snapshot_force",
-            stub_snapshot_create)
-        self.stubs.Set(volume.api.API, "delete_snapshot", stub_snapshot_delete)
-        self.stubs.Set(volume.api.API, "get_snapshot", stub_snapshot_get)
-        self.stubs.Set(volume.api.API, "get_all_snapshots",
-            stub_snapshot_get_all)
-        self.stubs.Set(volume.api.API, "get", fakes.stub_volume_get)
+        self.stubs.Set(cinder.API, "create_snapshot",
+                       fakes.stub_snapshot_create)
+        self.stubs.Set(cinder.API, "create_snapshot_force",
+                       fakes.stub_snapshot_create)
+        self.stubs.Set(cinder.API, "delete_snapshot",
+                       fakes.stub_snapshot_delete)
+        self.stubs.Set(cinder.API, "get_snapshot", fakes.stub_snapshot_get)
+        self.stubs.Set(cinder.API, "get_all_snapshots",
+                       fakes.stub_snapshot_get_all)
+        self.stubs.Set(cinder.API, "get", fakes.stub_volume_get)
+        self.flags(
+            osapi_compute_extension=[
+                'nova.api.openstack.compute.contrib.select_extensions'],
+            osapi_compute_ext_list=['Volumes'])
 
         self.context = context.get_admin_context()
+        self.app = fakes.wsgi_app(init_only=('os-snapshots',))
 
     def test_snapshot_create(self):
-        global _last_param
-        _last_param = {}
-
         snapshot = {"volume_id": 12,
                 "force": False,
                 "display_name": "Snapshot Test Name",
@@ -117,17 +59,9 @@ class SnapshotApiTest(test.TestCase):
         req.body = jsonutils.dumps(body)
         req.headers['content-type'] = 'application/json'
 
-        resp = req.get_response(fakes.wsgi_app())
-        LOG.debug(_("test_snapshot_create: param=%s"), _last_param)
+        resp = req.get_response(self.app)
         self.assertEqual(resp.status_int, 200)
-
-        # Compare if parameters were correctly passed to stub
-        self.assertEqual(_last_param['display_name'], "Snapshot Test Name")
-        self.assertEqual(_last_param['display_description'],
-            "Snapshot Test Desc")
-
         resp_dict = jsonutils.loads(resp.body)
-        LOG.debug(_("test_snapshot_create: resp_dict=%s"), resp_dict)
         self.assertTrue('snapshot' in resp_dict)
         self.assertEqual(resp_dict['snapshot']['displayName'],
                         snapshot['display_name'])
@@ -135,9 +69,6 @@ class SnapshotApiTest(test.TestCase):
                         snapshot['display_description'])
 
     def test_snapshot_create_force(self):
-        global _last_param
-        _last_param = {}
-
         snapshot = {"volume_id": 12,
                 "force": True,
                 "display_name": "Snapshot Test Name",
@@ -148,17 +79,10 @@ class SnapshotApiTest(test.TestCase):
         req.body = jsonutils.dumps(body)
         req.headers['content-type'] = 'application/json'
 
-        resp = req.get_response(fakes.wsgi_app())
-        LOG.debug(_("test_snapshot_create_force: param=%s"), _last_param)
+        resp = req.get_response(self.app)
         self.assertEqual(resp.status_int, 200)
 
-        # Compare if parameters were correctly passed to stub
-        self.assertEqual(_last_param['display_name'], "Snapshot Test Name")
-        self.assertEqual(_last_param['display_description'],
-            "Snapshot Test Desc")
-
         resp_dict = jsonutils.loads(resp.body)
-        LOG.debug(_("test_snapshot_create_force: resp_dict=%s"), resp_dict)
         self.assertTrue('snapshot' in resp_dict)
         self.assertEqual(resp_dict['snapshot']['displayName'],
                         snapshot['display_name'])
@@ -174,76 +98,56 @@ class SnapshotApiTest(test.TestCase):
         req.body = jsonutils.dumps(body)
         req.headers['content-type'] = 'application/json'
 
-        resp = req.get_response(fakes.wsgi_app())
-        LOG.debug(_("test_snapshot_create_force: param=%s"), _last_param)
+        resp = req.get_response(self.app)
         self.assertEqual(resp.status_int, 400)
 
     def test_snapshot_delete(self):
-        global _last_param
-        _last_param = {}
-
         snapshot_id = 123
         req = webob.Request.blank('/v2/fake/os-snapshots/%d' % snapshot_id)
         req.method = 'DELETE'
 
-        resp = req.get_response(fakes.wsgi_app())
+        resp = req.get_response(self.app)
         self.assertEqual(resp.status_int, 202)
-        self.assertEqual(str(_last_param['id']), str(snapshot_id))
 
     def test_snapshot_delete_invalid_id(self):
-        global _last_param
-        _last_param = {}
-
-        snapshot_id = 234
+        snapshot_id = -1
         req = webob.Request.blank('/v2/fake/os-snapshots/%d' % snapshot_id)
         req.method = 'DELETE'
 
-        resp = req.get_response(fakes.wsgi_app())
+        resp = req.get_response(self.app)
         self.assertEqual(resp.status_int, 404)
-        self.assertEqual(str(_last_param['snapshot_id']), str(snapshot_id))
 
     def test_snapshot_show(self):
-        global _last_param
-        _last_param = {}
-
         snapshot_id = 123
         req = webob.Request.blank('/v2/fake/os-snapshots/%d' % snapshot_id)
         req.method = 'GET'
-        resp = req.get_response(fakes.wsgi_app())
+        resp = req.get_response(self.app)
 
-        LOG.debug(_("test_snapshot_show: resp=%s"), resp)
         self.assertEqual(resp.status_int, 200)
-        self.assertEqual(str(_last_param['snapshot_id']), str(snapshot_id))
-
         resp_dict = jsonutils.loads(resp.body)
         self.assertTrue('snapshot' in resp_dict)
         self.assertEqual(resp_dict['snapshot']['id'], str(snapshot_id))
 
     def test_snapshot_show_invalid_id(self):
-        global _last_param
-        _last_param = {}
-
-        snapshot_id = 234
+        snapshot_id = -1
         req = webob.Request.blank('/v2/fake/os-snapshots/%d' % snapshot_id)
         req.method = 'GET'
-        resp = req.get_response(fakes.wsgi_app())
+        resp = req.get_response(self.app)
         self.assertEqual(resp.status_int, 404)
-        self.assertEqual(str(_last_param['snapshot_id']), str(snapshot_id))
 
     def test_snapshot_detail(self):
         req = webob.Request.blank('/v2/fake/os-snapshots/detail')
         req.method = 'GET'
-        resp = req.get_response(fakes.wsgi_app())
+        resp = req.get_response(self.app)
         self.assertEqual(resp.status_int, 200)
 
         resp_dict = jsonutils.loads(resp.body)
-        LOG.debug(_("test_snapshot_detail: resp_dict=%s"), resp_dict)
         self.assertTrue('snapshots' in resp_dict)
         resp_snapshots = resp_dict['snapshots']
-        self.assertEqual(len(resp_snapshots), 1)
+        self.assertEqual(len(resp_snapshots), 3)
 
         resp_snapshot = resp_snapshots.pop()
-        self.assertEqual(resp_snapshot['id'], 123)
+        self.assertEqual(resp_snapshot['id'], 102)
 
 
 class SnapshotSerializerTest(test.TestCase):
@@ -267,7 +171,6 @@ class SnapshotSerializerTest(test.TestCase):
             )
         text = serializer.serialize(dict(snapshot=raw_snapshot))
 
-        print text
         tree = etree.fromstring(text)
 
         self._verify_snapshot(raw_snapshot, tree)
@@ -294,7 +197,6 @@ class SnapshotSerializerTest(test.TestCase):
                 )]
         text = serializer.serialize(dict(snapshots=raw_snapshots))
 
-        print text
         tree = etree.fromstring(text)
 
         self.assertEqual('snapshots', tree.tag)
