@@ -1,4 +1,4 @@
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -30,6 +30,8 @@ from nova.api.openstack import xmlutil
 import nova.context
 from nova.openstack.common import jsonutils
 from nova import test
+from nova.tests.api.openstack import fakes
+from nova.tests import matchers
 
 
 TEST_LIMITS = [
@@ -75,6 +77,7 @@ class LimitsControllerTest(BaseLimitTestSuite):
         """Run before each test."""
         super(LimitsControllerTest, self).setUp()
         self.controller = limits.create_resource()
+        self.ctrler = limits.LimitsController()
 
     def _get_index_request(self, accept_header="application/json"):
         """Helper to set routing arguments."""
@@ -100,7 +103,7 @@ class LimitsControllerTest(BaseLimitTestSuite):
         return request
 
     def test_empty_index_json(self):
-        """Test getting empty limit details in JSON."""
+        # Test getting empty limit details in JSON.
         request = self._get_index_request()
         response = request.get_response(self.controller)
         expected = {
@@ -113,15 +116,13 @@ class LimitsControllerTest(BaseLimitTestSuite):
         self.assertEqual(expected, body)
 
     def test_index_json(self):
-        """Test getting limit details in JSON."""
+        # Test getting limit details in JSON.
         request = self._get_index_request()
         request = self._populate_limits(request)
         self.absolute_limits = {
             'ram': 512,
             'instances': 5,
             'cores': 21,
-            'gigabytes': 512,
-            'volumes': 5,
             'key_pairs': 10,
             'floating_ips': 10,
             'security_groups': 10,
@@ -170,8 +171,6 @@ class LimitsControllerTest(BaseLimitTestSuite):
                     "maxTotalRAMSize": 512,
                     "maxTotalInstances": 5,
                     "maxTotalCores": 21,
-                    "maxTotalVolumeGigabytes": 512,
-                    "maxTotalVolumes": 5,
                     "maxTotalKeypairs": 10,
                     "maxTotalFloatingIps": 10,
                     "maxSecurityGroups": 10,
@@ -192,7 +191,7 @@ class LimitsControllerTest(BaseLimitTestSuite):
         return request
 
     def test_index_diff_regex(self):
-        """Test getting limit details in JSON."""
+        # Test getting limit details in JSON.
         request = self._get_index_request()
         request = self._populate_limits_diff_regex(request)
         response = request.get_response(self.controller)
@@ -287,6 +286,31 @@ class LimitsControllerTest(BaseLimitTestSuite):
         }
         self._test_index_absolute_limits_json(expected)
 
+    def test_limit_create(self):
+        req = fakes.HTTPRequest.blank('/v2/fake/limits')
+        self.assertRaises(webob.exc.HTTPNotImplemented, self.ctrler.create,
+                          req, {})
+
+    def test_limit_delete(self):
+        req = fakes.HTTPRequest.blank('/v2/fake/limits')
+        self.assertRaises(webob.exc.HTTPNotImplemented, self.ctrler.delete,
+                          req, 1)
+
+    def test_limit_detail(self):
+        req = fakes.HTTPRequest.blank('/v2/fake/limits')
+        self.assertRaises(webob.exc.HTTPNotImplemented, self.ctrler.detail,
+                          req)
+
+    def test_limit_show(self):
+        req = fakes.HTTPRequest.blank('/v2/fake/limits')
+        self.assertRaises(webob.exc.HTTPNotImplemented, self.ctrler.show,
+                          req, 1)
+
+    def test_limit_update(self):
+        req = fakes.HTTPRequest.blank('/v2/fake/limits')
+        self.assertRaises(webob.exc.HTTPNotImplemented, self.ctrler.update,
+                          req, 1, {})
+
 
 class TestLimiter(limits.Limiter):
     pass
@@ -311,17 +335,17 @@ class LimitMiddlewareTest(BaseLimitTestSuite):
                                                  self.__class__.__module__)
 
     def test_limit_class(self):
-        """Test that middleware selected correct limiter class."""
+        # Test that middleware selected correct limiter class.
         assert isinstance(self.app._limiter, TestLimiter)
 
     def test_good_request(self):
-        """Test successful GET request through middleware."""
+        # Test successful GET request through middleware.
         request = webob.Request.blank("/")
         response = request.get_response(self.app)
         self.assertEqual(200, response.status_int)
 
     def test_limited_request_json(self):
-        """Test a rate-limited (413) GET request through middleware."""
+        # Test a rate-limited (413) GET request through middleware.
         request = webob.Request.blank("/")
         response = request.get_response(self.app)
         self.assertEqual(200, response.status_int)
@@ -344,7 +368,7 @@ class LimitMiddlewareTest(BaseLimitTestSuite):
         self.assertEqual(retryAfter, "60")
 
     def test_limited_request_xml(self):
-        """Test a rate-limited (413) response as XML"""
+        # Test a rate-limited (413) response as XML.
         request = webob.Request.blank("/")
         response = request.get_response(self.app)
         self.assertEqual(200, response.status_int)
@@ -374,7 +398,7 @@ class LimitTest(BaseLimitTestSuite):
     """
 
     def test_GET_no_delay(self):
-        """Test a limit handles 1 GET per second."""
+        # Test a limit handles 1 GET per second.
         limit = limits.Limit("GET", "*", ".*", 1, 1)
         delay = limit("GET", "/anything")
         self.assertEqual(None, delay)
@@ -382,7 +406,7 @@ class LimitTest(BaseLimitTestSuite):
         self.assertEqual(0, limit.last_request)
 
     def test_GET_delay(self):
-        """Test two calls to 1 GET per second limit."""
+        # Test two calls to 1 GET per second limit.
         limit = limits.Limit("GET", "*", ".*", 1, 1)
         delay = limit("GET", "/anything")
         self.assertEqual(None, delay)
@@ -407,32 +431,32 @@ class ParseLimitsTest(BaseLimitTestSuite):
     """
 
     def test_invalid(self):
-        """Test that parse_limits() handles invalid input correctly."""
+        # Test that parse_limits() handles invalid input correctly.
         self.assertRaises(ValueError, limits.Limiter.parse_limits,
                           ';;;;;')
 
     def test_bad_rule(self):
-        """Test that parse_limits() handles bad rules correctly."""
+        # Test that parse_limits() handles bad rules correctly.
         self.assertRaises(ValueError, limits.Limiter.parse_limits,
                           'GET, *, .*, 20, minute')
 
     def test_missing_arg(self):
-        """Test that parse_limits() handles missing args correctly."""
+        # Test that parse_limits() handles missing args correctly.
         self.assertRaises(ValueError, limits.Limiter.parse_limits,
                           '(GET, *, .*, 20)')
 
     def test_bad_value(self):
-        """Test that parse_limits() handles bad values correctly."""
+        # Test that parse_limits() handles bad values correctly.
         self.assertRaises(ValueError, limits.Limiter.parse_limits,
                           '(GET, *, .*, foo, minute)')
 
     def test_bad_unit(self):
-        """Test that parse_limits() handles bad units correctly."""
+        # Test that parse_limits() handles bad units correctly.
         self.assertRaises(ValueError, limits.Limiter.parse_limits,
                           '(GET, *, .*, 20, lightyears)')
 
     def test_multiple_rules(self):
-        """Test that parse_limits() handles multiple rules correctly."""
+        # Test that parse_limits() handles multiple rules correctly.
         try:
             l = limits.Limiter.parse_limits('(get, *, .*, 20, minute);'
                                             '(PUT, /foo*, /foo.*, 10, hour);'
@@ -496,9 +520,7 @@ class LimiterTest(BaseLimitTestSuite):
         self.assertEqual(delay, (None, None))
 
     def test_no_delay_PUT(self):
-        """
-        Simple test to ensure no delay on a single call for a known limit.
-        """
+        # Simple test to ensure no delay on a single call for a known limit.
         delay = self.limiter.check_for_delay("PUT", "/anything")
         self.assertEqual(delay, (None, None))
 
@@ -526,9 +548,7 @@ class LimiterTest(BaseLimitTestSuite):
         self.failUnlessAlmostEqual(expected, results, 8)
 
     def test_delay_GET(self):
-        """
-        Ensure the 11th GET will result in NO delay.
-        """
+        # Ensure the 11th GET will result in NO delay.
         expected = [None] * 11
         results = list(self._check(11, "GET", "/anything"))
 
@@ -567,9 +587,7 @@ class LimiterTest(BaseLimitTestSuite):
         self.assertEqual(expected, results)
 
     def test_multiple_delays(self):
-        """
-        Ensure multiple requests still get a delay.
-        """
+        # Ensure multiple requests still get a delay.
         expected = [None] * 10 + [6.0] * 10
         results = list(self._check(20, "PUT", "/anything"))
         self.assertEqual(expected, results)
@@ -581,15 +599,11 @@ class LimiterTest(BaseLimitTestSuite):
         self.assertEqual(expected, results)
 
     def test_user_limit(self):
-        """
-        Test user-specific limits.
-        """
+        # Test user-specific limits.
         self.assertEqual(self.limiter.levels['user3'], [])
 
     def test_multiple_users(self):
-        """
-        Tests involving multiple users.
-        """
+        # Tests involving multiple users.
         # User1
         expected = [None] * 10 + [6.0] * 10
         results = list(self._check(20, "PUT", "/anything", "user1"))
@@ -631,7 +645,7 @@ class WsgiLimiterTest(BaseLimitTestSuite):
         self.app = limits.WsgiLimiter(TEST_LIMITS)
 
     def _request_data(self, verb, path):
-        """Get data decribing a limit request verb/path."""
+        """Get data describing a limit request verb/path."""
         return jsonutils.dumps({"verb": verb, "path": path})
 
     def _request(self, verb, url, username=None):
@@ -655,7 +669,7 @@ class WsgiLimiterTest(BaseLimitTestSuite):
         self.assertEqual(response.status_int, 204)
 
     def test_invalid_methods(self):
-        """Only POSTs should work."""
+        # Only POSTs should work.
         requests = []
         for method in ["GET", "PUT", "DELETE", "HEAD", "OPTIONS"]:
             request = webob.Request.blank("/", method=method)
@@ -797,12 +811,12 @@ class WsgiLimiterProxyTest(BaseLimitTestSuite):
         self.proxy = limits.WsgiLimiterProxy("169.254.0.1:80")
 
     def test_200(self):
-        """Successful request test."""
+        # Successful request test.
         delay = self.proxy.check_for_delay("GET", "/anything")
         self.assertEqual(delay, (None, None))
 
     def test_403(self):
-        """Forbidden request test."""
+        # Forbidden request test.
         delay = self.proxy.check_for_delay("GET", "/delayed")
         self.assertEqual(delay, (None, None))
 
@@ -866,7 +880,7 @@ class LimitsViewBuilderTest(test.TestCase):
 
         output = self.view_builder.build(self.rate_limits,
                                          self.absolute_limits)
-        self.assertDictMatch(output, expected_limits)
+        self.assertThat(output, matchers.DictMatches(expected_limits))
 
     def test_build_limits_empty_limits(self):
         expected_limits = {"limits": {"rate": [],
@@ -875,7 +889,7 @@ class LimitsViewBuilderTest(test.TestCase):
         abs_limits = {}
         rate_limits = []
         output = self.view_builder.build(rate_limits, abs_limits)
-        self.assertDictMatch(output, expected_limits)
+        self.assertThat(output, matchers.DictMatches(expected_limits))
 
 
 class LimitsXMLSerializationTest(test.TestCase):
@@ -887,7 +901,6 @@ class LimitsXMLSerializationTest(test.TestCase):
                    "absolute": {}}}
 
         output = serializer.serialize(fixture)
-        print output
         has_dec = output.startswith("<?xml version='1.0' encoding='UTF-8'?>")
         self.assertTrue(has_dec)
 
@@ -918,7 +931,6 @@ class LimitsXMLSerializationTest(test.TestCase):
                                  "maxPersonalitySize": 10240}}}
 
         output = serializer.serialize(fixture)
-        print output
         root = etree.XML(output)
         xmlutil.validate_schema(root, 'limits')
 
@@ -953,7 +965,6 @@ class LimitsXMLSerializationTest(test.TestCase):
                    "absolute": {}}}
 
         output = serializer.serialize(fixture)
-        print output
         root = etree.XML(output)
         xmlutil.validate_schema(root, 'limits')
 
