@@ -111,6 +111,30 @@ class ConsoleauthTestCase(test.TestCase):
         port = self.manager.get_backdoor_port(self.context)
         self.assertEqual(port, self.manager.backdoor_port)
 
+    def test_delete_expired_tokens(self):
+        instance = u"12345"
+        self.useFixture(test.TimeOverride())
+        token = u'mytok'
+        self.flags(console_token_ttl=1)
+
+        self._stub_validate_console_port(True)
+
+        self.manager.authorize_console(self.context, token, 'novnc',
+                                         '127.0.0.1', '8080', 'host',
+                                         instance)
+        timeutils.advance_time_seconds(1)
+        self.assertFalse(self.manager.check_token(self.context, token))
+
+        token1 = u'mytok2'
+        self.manager.authorize_console(self.context, token1, 'novnc',
+                                       '127.0.0.1', '8080', 'host',
+                                       instance)
+        stored_tokens = self.manager._get_tokens_for_instance(
+                instance)
+        # when trying to store token1, expired token is removed fist.
+        self.assertTrue(len(stored_tokens), 1)
+        self.assertTrue(stored_tokens[0], token1)
+
 
 class ControlauthMemcacheEncodingTestCase(test.TestCase):
     def setUp(self):
