@@ -18,61 +18,70 @@
 Unit Tests for nova.consoleauth.rpcapi
 """
 
+from oslo.config import cfg
+
 from nova.consoleauth import rpcapi as consoleauth_rpcapi
 from nova import context
-from nova import flags
 from nova.openstack.common import rpc
 from nova import test
 
-
-FLAGS = flags.FLAGS
+CONF = cfg.CONF
 
 
 class ConsoleAuthRpcAPITestCase(test.TestCase):
-
-    def setUp(self):
-        super(ConsoleAuthRpcAPITestCase, self).setUp()
-
-    def tearDown(self):
-        super(ConsoleAuthRpcAPITestCase, self).tearDown()
-
     def _test_consoleauth_api(self, method, **kwargs):
+        do_cast = kwargs.pop('_do_cast', False)
         ctxt = context.RequestContext('fake_user', 'fake_project')
         rpcapi = consoleauth_rpcapi.ConsoleAuthAPI()
         expected_retval = 'foo'
+        expected_version = kwargs.pop('version', rpcapi.BASE_RPC_API_VERSION)
         expected_msg = rpcapi.make_msg(method, **kwargs)
-        expected_msg['version'] = rpcapi.BASE_RPC_API_VERSION
+        expected_msg['version'] = expected_version
+
+        if method == 'get_backdoor_port':
+            del expected_msg['args']['host']
 
         self.call_ctxt = None
         self.call_topic = None
         self.call_msg = None
         self.call_timeout = None
 
-        def _fake_call(_ctxt, _topic, _msg, _timeout):
+        def _fake_call(_ctxt, _topic, _msg, _timeout=None):
             self.call_ctxt = _ctxt
             self.call_topic = _topic
             self.call_msg = _msg
             self.call_timeout = _timeout
             return expected_retval
 
-        self.stubs.Set(rpc, 'call', _fake_call)
+        if do_cast:
+            self.stubs.Set(rpc, 'cast', _fake_call)
+        else:
+            self.stubs.Set(rpc, 'call', _fake_call)
 
         retval = getattr(rpcapi, method)(ctxt, **kwargs)
 
-        self.assertEqual(retval, expected_retval)
+        if not do_cast:
+            self.assertEqual(retval, expected_retval)
         self.assertEqual(self.call_ctxt, ctxt)
-        self.assertEqual(self.call_topic, FLAGS.consoleauth_topic)
+        self.assertEqual(self.call_topic, CONF.consoleauth_topic)
         self.assertEqual(self.call_msg, expected_msg)
         self.assertEqual(self.call_timeout, None)
 
     def test_authorize_console(self):
         self._test_consoleauth_api('authorize_console', token='token',
                 console_type='ctype', host='h', port='p',
-                internal_access_path='iap', instance_uuid="1234")
+                internal_access_path='iap', instance_uuid="instance",
+                version="1.2")
 
     def test_check_token(self):
         self._test_consoleauth_api('check_token', token='t')
 
     def test_delete_tokens_for_instnace(self):
         self._test_consoleauth_api('delete_tokens_for_instance',
-                                   instance_uuid="instance")
+                                   _do_cast=True,
+                                   instance_uuid="instance",
+                                   version='1.2')
+
+    def test_get_backdoor_port(self):
+        self._test_consoleauth_api('get_backdoor_port', host='fake_host',
+                                   version='1.1')
