@@ -16,29 +16,70 @@
 #    under the License.
 
 import time
-import unittest
+import zlib
 
-from nova.log import logging
-from nova.tests.integrated import integrated_helpers
+from nova.openstack.common import log as logging
+from nova.tests import fake_network
 from nova.tests.integrated.api import client
+from nova.tests.integrated import integrated_helpers
+import nova.virt.fake
 
 
-LOG = logging.getLogger('nova.tests.integrated')
+LOG = logging.getLogger(__name__)
 
 
 class ServersTest(integrated_helpers._IntegratedTestBase):
+    def setUp(self):
+        super(ServersTest, self).setUp()
+        self.conductor = self.start_service(
+            'conductor', manager='nova.conductor.manager.ConductorManager')
+
+    def _wait_for_state_change(self, server, from_status):
+        for i in xrange(0, 50):
+            server = self.api.get_server(server['id'])
+            if server['status'] != from_status:
+                break
+            time.sleep(.1)
+
+        return server
+
+    def _restart_compute_service(self, *args, **kwargs):
+        """restart compute service. NOTE: fake driver forgets all instances."""
+        self.compute.kill()
+        self.compute = self.start_service('compute', *args, **kwargs)
 
     def test_get_servers(self):
-        """Simple check that listing servers works."""
+        # Simple check that listing servers works.
         servers = self.api.get_servers()
         for server in servers:
             LOG.debug("server: %s" % server)
 
+    def test_create_server_with_error(self):
+        # Create a server which will enter error state.
+        fake_network.set_stub_network_methods(self.stubs)
+
+        def throw_error(*_):
+            raise Exception()
+
+        self.stubs.Set(nova.virt.fake.FakeDriver, 'spawn', throw_error)
+
+        server = self._build_minimal_create_server_request()
+        created_server = self.api.post_server({"server": server})
+        created_server_id = created_server['id']
+
+        found_server = self.api.get_server(created_server_id)
+        self.assertEqual(created_server_id, found_server['id'])
+
+        found_server = self._wait_for_state_change(found_server, 'BUILD')
+
+        self.assertEqual('ERROR', found_server['status'])
+        self._delete_server(created_server_id)
+
     def test_create_and_delete_server(self):
-        """Creates and deletes a server."""
+        # Creates and deletes a server.
+        fake_network.set_stub_network_methods(self.stubs)
 
         # Create server
-
         # Build the server data gradually, checking errors along the way
         server = {}
         good_server = self._build_minimal_create_server_request()
@@ -56,18 +97,15 @@ class ServersTest(integrated_helpers._IntegratedTestBase):
         self.assertRaises(client.OpenStackApiException,
                           self.api.post_server, post)
 
-        # Add a valid imageId/imageRef
-        server['imageId'] = good_server.get('imageId')
+        # Add a valid imageRef
         server['imageRef'] = good_server.get('imageRef')
 
-        # Without flavorId, this throws 500
+        # Without flavorRef, this throws 500
         # TODO(justinsb): Check whatever the spec says should be thrown here
         self.assertRaises(client.OpenStackApiException,
                           self.api.post_server, post)
 
-        # Set a valid flavorId/flavorRef
         server['flavorRef'] = good_server.get('flavorRef')
-        server['flavorId'] = good_server.get('flavorId')
 
         # Without a name, this throws 500
         # TODO(justinsb): Check whatever the spec says should be thrown here
@@ -91,19 +129,10 @@ class ServersTest(integrated_helpers._IntegratedTestBase):
         server_ids = [server['id'] for server in servers]
         self.assertTrue(created_server_id in server_ids)
 
-        # Wait (briefly) for creation
-        retries = 0
-        while found_server['status'] == 'build':
-            LOG.debug("found server: %s" % found_server)
-            time.sleep(1)
-            found_server = self.api.get_server(created_server_id)
-            retries = retries + 1
-            if retries > 5:
-                break
-
+        found_server = self._wait_for_state_change(found_server, 'BUILD')
         # It should be available...
         # TODO(justinsb): Mock doesn't yet do this...
-        #self.assertEqual('available', found_server['status'])
+        self.assertEqual('ACTIVE', found_server['status'])
         servers = self.api.get_servers(detail=True)
         for server in servers:
             self.assertTrue("image" in server)
@@ -111,12 +140,117 @@ class ServersTest(integrated_helpers._IntegratedTestBase):
 
         self._delete_server(created_server_id)
 
-    def _delete_server(self, server_id):
-        # Delete the server
-        self.api.delete_server(server_id)
+    def test_deferred_delete(self):
+        # Creates, deletes and waits for server to be reclaimed.
+        self.flags(reclaim_instance_interval=1)
+        fake_network.set_stub_network_methods(self.stubs)
 
+        # enforce periodic tasks run in short time to avoid wait for 60s.
+        self._restart_compute_service(periodic_interval_max=0.3,
+                                      periodic_fuzzy_delay=0)
+
+        # Create server
+        server = self._build_minimal_create_server_request()
+
+        created_server = self.api.post_server({'server': server})
+        LOG.debug("created_server: %s" % created_server)
+        self.assertTrue(created_server['id'])
+        created_server_id = created_server['id']
+
+        # Wait for it to finish being created
+        found_server = self._wait_for_state_change(created_server, 'BUILD')
+
+        # It should be available...
+        self.assertEqual('ACTIVE', found_server['status'])
+
+        # Cannot restore unless instance is deleted
+        self.assertRaises(client.OpenStackApiException,
+                          self.api.post_server_action, created_server_id,
+                          {'restore': {}})
+
+        # Cannot forceDelete unless instance is deleted
+        self.assertRaises(client.OpenStackApiException,
+                          self.api.post_server_action, created_server_id,
+                          {'forceDelete': {}})
+
+        # Delete the server
+        self.api.delete_server(created_server_id)
+
+        # Wait for queued deletion
+        found_server = self._wait_for_state_change(found_server, 'ACTIVE')
+        self.assertEqual('DELETED', found_server['status'])
+
+        # Wait for real deletion
+        self._wait_for_deletion(created_server_id)
+
+    def test_deferred_delete_restore(self):
+        # Creates, deletes and restores a server.
+        self.flags(reclaim_instance_interval=3600)
+        fake_network.set_stub_network_methods(self.stubs)
+
+        # Create server
+        server = self._build_minimal_create_server_request()
+
+        created_server = self.api.post_server({'server': server})
+        LOG.debug("created_server: %s" % created_server)
+        self.assertTrue(created_server['id'])
+        created_server_id = created_server['id']
+
+        # Wait for it to finish being created
+        found_server = self._wait_for_state_change(created_server, 'BUILD')
+
+        # It should be available...
+        self.assertEqual('ACTIVE', found_server['status'])
+
+        # Delete the server
+        self.api.delete_server(created_server_id)
+
+        # Wait for queued deletion
+        found_server = self._wait_for_state_change(found_server, 'ACTIVE')
+        self.assertEqual('DELETED', found_server['status'])
+
+        # Restore server
+        self.api.post_server_action(created_server_id, {'restore': {}})
+
+        # Wait for server to become active again
+        found_server = self._wait_for_state_change(found_server, 'DELETED')
+        self.assertEqual('ACTIVE', found_server['status'])
+
+    def test_deferred_delete_force(self):
+        # Creates, deletes and force deletes a server.
+        self.flags(reclaim_instance_interval=3600)
+        fake_network.set_stub_network_methods(self.stubs)
+
+        # Create server
+        server = self._build_minimal_create_server_request()
+
+        created_server = self.api.post_server({'server': server})
+        LOG.debug("created_server: %s" % created_server)
+        self.assertTrue(created_server['id'])
+        created_server_id = created_server['id']
+
+        # Wait for it to finish being created
+        found_server = self._wait_for_state_change(created_server, 'BUILD')
+
+        # It should be available...
+        self.assertEqual('ACTIVE', found_server['status'])
+
+        # Delete the server
+        self.api.delete_server(created_server_id)
+
+        # Wait for queued deletion
+        found_server = self._wait_for_state_change(found_server, 'ACTIVE')
+        self.assertEqual('DELETED', found_server['status'])
+
+        # Force delete server
+        self.api.post_server_action(created_server_id, {'forceDelete': {}})
+
+        # Wait for real deletion
+        self._wait_for_deletion(created_server_id)
+
+    def _wait_for_deletion(self, server_id):
         # Wait (briefly) for deletion
-        for _retries in range(5):
+        for _retries in range(50):
             try:
                 found_server = self.api.get_server(server_id)
             except client.OpenStackApiNotFoundException:
@@ -129,13 +263,19 @@ class ServersTest(integrated_helpers._IntegratedTestBase):
             # TODO(justinsb): Mock doesn't yet do accurate state changes
             #if found_server['status'] != 'deleting':
             #    break
-            time.sleep(1)
+            time.sleep(.1)
 
         # Should be gone
         self.assertFalse(found_server)
 
+    def _delete_server(self, server_id):
+        # Delete the server
+        self.api.delete_server(server_id)
+        self._wait_for_deletion(server_id)
+
     def test_create_server_with_metadata(self):
-        """Creates a server with metadata."""
+        # Creates a server with metadata.
+        fake_network.set_stub_network_methods(self.stubs)
 
         # Build the server data gradually, checking errors along the way
         server = self._build_minimal_create_server_request()
@@ -152,10 +292,6 @@ class ServersTest(integrated_helpers._IntegratedTestBase):
         self.assertTrue(created_server['id'])
         created_server_id = created_server['id']
 
-        # Reenable when bug fixed
-        self.assertEqual(metadata, created_server.get('metadata'))
-        # Check it's there
-
         found_server = self.api.get_server(created_server_id)
         self.assertEqual(created_server_id, found_server['id'])
         self.assertEqual(metadata, found_server.get('metadata'))
@@ -180,74 +316,8 @@ class ServersTest(integrated_helpers._IntegratedTestBase):
         self._delete_server(created_server_id)
 
     def test_create_and_rebuild_server(self):
-        """Rebuild a server."""
-
-        # create a server with initially has no metadata
-        server = self._build_minimal_create_server_request()
-        server_post = {'server': server}
-        created_server = self.api.post_server(server_post)
-        LOG.debug("created_server: %s" % created_server)
-        self.assertTrue(created_server['id'])
-        created_server_id = created_server['id']
-
-        # rebuild the server with metadata
-        post = {}
-        post['rebuild'] = {
-            "imageRef": "https://localhost/v1.1/32278/images/3",
-            "name": "blah",
-        }
-
-        self.api.post_server_action(created_server_id, post)
-        LOG.debug("rebuilt server: %s" % created_server)
-        self.assertTrue(created_server['id'])
-
-        found_server = self.api.get_server(created_server_id)
-        self.assertEqual(created_server_id, found_server['id'])
-        self.assertEqual({}, found_server.get('metadata'))
-        self.assertEqual('blah', found_server.get('name'))
-        self.assertEqual('3', found_server.get('image')['id'])
-
-        # Cleanup
-        self._delete_server(created_server_id)
-
-    def test_create_and_rebuild_server_with_metadata(self):
-        """Rebuild a server with metadata."""
-
-        # create a server with initially has no metadata
-        server = self._build_minimal_create_server_request()
-        server_post = {'server': server}
-        created_server = self.api.post_server(server_post)
-        LOG.debug("created_server: %s" % created_server)
-        self.assertTrue(created_server['id'])
-        created_server_id = created_server['id']
-
-        # rebuild the server with metadata
-        post = {}
-        post['rebuild'] = {
-            "imageRef": "https://localhost/v1.1/32278/images/2",
-            "name": "blah",
-        }
-
-        metadata = {}
-        for i in range(30):
-            metadata['key_%s' % i] = 'value_%s' % i
-
-        post['rebuild']['metadata'] = metadata
-
-        self.api.post_server_action(created_server_id, post)
-        LOG.debug("rebuilt server: %s" % created_server)
-        self.assertTrue(created_server['id'])
-
-        found_server = self.api.get_server(created_server_id)
-        self.assertEqual(created_server_id, found_server['id'])
-        self.assertEqual(metadata, found_server.get('metadata'))
-        self.assertEqual('blah', found_server.get('name'))
-
-        # Cleanup
-        self._delete_server(created_server_id)
-
-    def test_create_and_rebuild_server_with_metadata_removal(self):
-        """Rebuild a server with metadata."""
+        # Rebuild a server with metadata.
+        fake_network.set_stub_network_methods(self.stubs)
 
         # create a server with initially has no metadata
         server = self._build_minimal_create_server_request()
@@ -264,30 +334,57 @@ class ServersTest(integrated_helpers._IntegratedTestBase):
         self.assertTrue(created_server['id'])
         created_server_id = created_server['id']
 
-        # rebuild the server with metadata
+        created_server = self._wait_for_state_change(created_server, 'BUILD')
+
+        # rebuild the server with metadata and other server attributes
         post = {}
         post['rebuild'] = {
-            "imageRef": "https://localhost/v1.1/32278/images/2",
+            "imageRef": "76fa36fc-c930-4bf3-8c8a-ea2a2420deb6",
             "name": "blah",
+            "accessIPv4": "172.19.0.2",
+            "accessIPv6": "fe80::2",
+            "metadata": {'some': 'thing'},
         }
 
-        metadata = {}
-        post['rebuild']['metadata'] = metadata
-
         self.api.post_server_action(created_server_id, post)
         LOG.debug("rebuilt server: %s" % created_server)
         self.assertTrue(created_server['id'])
 
         found_server = self.api.get_server(created_server_id)
         self.assertEqual(created_server_id, found_server['id'])
-        self.assertEqual(metadata, found_server.get('metadata'))
+        self.assertEqual({'some': 'thing'}, found_server.get('metadata'))
         self.assertEqual('blah', found_server.get('name'))
+        self.assertEqual(post['rebuild']['imageRef'],
+                         found_server.get('image')['id'])
+        self.assertEqual('172.19.0.2', found_server['accessIPv4'])
+        self.assertEqual('fe80::2', found_server['accessIPv6'])
+
+        # rebuild the server with empty metadata and nothing else
+        post = {}
+        post['rebuild'] = {
+            "imageRef": "76fa36fc-c930-4bf3-8c8a-ea2a2420deb6",
+            "metadata": {},
+        }
+
+        self.api.post_server_action(created_server_id, post)
+        LOG.debug("rebuilt server: %s" % created_server)
+        self.assertTrue(created_server['id'])
+
+        found_server = self.api.get_server(created_server_id)
+        self.assertEqual(created_server_id, found_server['id'])
+        self.assertEqual({}, found_server.get('metadata'))
+        self.assertEqual('blah', found_server.get('name'))
+        self.assertEqual(post['rebuild']['imageRef'],
+                         found_server.get('image')['id'])
+        self.assertEqual('172.19.0.2', found_server['accessIPv4'])
+        self.assertEqual('fe80::2', found_server['accessIPv6'])
 
         # Cleanup
         self._delete_server(created_server_id)
 
     def test_rename_server(self):
-        """Test building and renaming a server."""
+        # Test building and renaming a server.
+        fake_network.set_stub_network_methods(self.stubs)
 
         # Create a server
         server = self._build_minimal_create_server_request()
@@ -306,6 +403,78 @@ class ServersTest(integrated_helpers._IntegratedTestBase):
         # Cleanup
         self._delete_server(server_id)
 
+    def test_create_multiple_servers(self):
+        # Creates multiple servers and checks for reservation_id.
 
-if __name__ == "__main__":
-    unittest.main()
+        # Create 2 servers, setting 'return_reservation_id, which should
+        # return a reservation_id
+        server = self._build_minimal_create_server_request()
+        server['min_count'] = 2
+        server['return_reservation_id'] = True
+        post = {'server': server}
+        response = self.api.post_server(post)
+        self.assertIn('reservation_id', response)
+        reservation_id = response['reservation_id']
+        self.assertNotIn(reservation_id, ['', None])
+
+        # Create 1 more server, which should not return a reservation_id
+        server = self._build_minimal_create_server_request()
+        post = {'server': server}
+        created_server = self.api.post_server(post)
+        self.assertTrue(created_server['id'])
+        created_server_id = created_server['id']
+
+        # lookup servers created by the first request.
+        servers = self.api.get_servers(detail=True,
+                search_opts={'reservation_id': reservation_id})
+        server_map = dict((server['id'], server) for server in servers)
+        found_server = server_map.get(created_server_id)
+        # The server from the 2nd request should not be there.
+        self.assertEqual(found_server, None)
+        # Should have found 2 servers.
+        self.assertEqual(len(server_map), 2)
+
+        # Cleanup
+        self._delete_server(created_server_id)
+        for server_id in server_map.iterkeys():
+            self._delete_server(server_id)
+
+    def test_create_server_with_injected_files(self):
+        # Creates a server with injected_files.
+        fake_network.set_stub_network_methods(self.stubs)
+        personality = []
+
+        # Inject a text file
+        data = 'Hello, World!'
+        personality.append({
+            'path': '/helloworld.txt',
+            'contents': data.encode('base64'),
+        })
+
+        # Inject a binary file
+        data = zlib.compress('Hello, World!')
+        personality.append({
+            'path': '/helloworld.zip',
+            'contents': data.encode('base64'),
+        })
+
+        # Create server
+        server = self._build_minimal_create_server_request()
+        server['personality'] = personality
+
+        post = {'server': server}
+
+        created_server = self.api.post_server(post)
+        LOG.debug("created_server: %s" % created_server)
+        self.assertTrue(created_server['id'])
+        created_server_id = created_server['id']
+
+        # Check it's there
+        found_server = self.api.get_server(created_server_id)
+        self.assertEqual(created_server_id, found_server['id'])
+
+        found_server = self._wait_for_state_change(found_server, 'BUILD')
+        self.assertEqual('ACTIVE', found_server['status'])
+
+        # Cleanup
+        self._delete_server(created_server_id)
