@@ -139,6 +139,7 @@ class _ImageTestCase(object):
             return
 
         self.stubs.Set(os.path, 'exists', lambda _: True)
+        self.stubs.Set(os, 'access', lambda p, w: True)
 
         # Call twice to verify testing fallocate is only called once.
         image.cache(fake_fetch, self.TEMPLATE_PATH, self.SIZE)
@@ -149,6 +150,25 @@ class _ImageTestCase(object):
              'fallocate -n -l %s %s' % (self.SIZE, self.PATH),
              'fallocate -n -l %s %s' % (self.SIZE, self.PATH)])
 
+    def test_prealloc_image_without_write_access(self):
+        CONF.set_override('preallocate_images', 'space')
+
+        fake_utils.fake_execute_clear_log()
+        fake_utils.stub_out_utils_execute(self.stubs)
+        image = self.image_class(self.INSTANCE, self.NAME)
+
+        def fake_fetch(target, *args, **kwargs):
+            return
+
+        self.stubs.Set(image, '_can_fallocate', lambda: True)
+        self.stubs.Set(os.path, 'exists', lambda _: True)
+        self.stubs.Set(os, 'access', lambda p, w: False)
+
+        # Testing fallocate is only called when user has write access.
+        image.cache(fake_fetch, self.TEMPLATE_PATH, self.SIZE)
+
+        self.assertEqual(fake_utils.fake_execute_get_log(), [])
+
 
 class RawTestCase(_ImageTestCase, test.TestCase):
 
