@@ -15,18 +15,15 @@
 #    under the License.
 
 from lxml import etree
-from lxml import objectify
 
 from nova import test
-
+from nova.tests import matchers
 from nova.virt.libvirt import config
 
 
 class LibvirtConfigBaseTest(test.TestCase):
     def assertXmlEqual(self, expectedXmlstr, actualXmlstr):
-        expected = etree.tostring(objectify.fromstring(expectedXmlstr))
-        actual = etree.tostring(objectify.fromstring(actualXmlstr))
-        self.assertEqual(expected, actual)
+        self.assertThat(actualXmlstr, matchers.XMLMatches(expectedXmlstr))
 
 
 class LibvirtConfigTest(LibvirtConfigBaseTest):
@@ -65,6 +62,7 @@ class LibvirtConfigCapsTest(LibvirtConfigBaseTest):
         xmlin = """
         <capabilities>
           <host>
+            <uuid>c7a5fdbd-edaf-9455-926a-d65c16db1809</uuid>
             <cpu>
               <arch>x86_64</arch>
               <model>Opteron_G3</model>
@@ -88,6 +86,7 @@ class LibvirtConfigCapsTest(LibvirtConfigBaseTest):
         obj.parse_str(xmlin)
 
         self.assertEqual(type(obj.host), config.LibvirtConfigCapsHost)
+        self.assertEqual(obj.host.uuid, "c7a5fdbd-edaf-9455-926a-d65c16db1809")
 
         xmlout = obj.to_xml()
 
@@ -303,6 +302,85 @@ class LibvirtConfigGuestCPUTest(LibvirtConfigBaseTest):
         """)
 
 
+class LibvirtConfigGuestSMBIOSTest(LibvirtConfigBaseTest):
+
+    def test_config_simple(self):
+        obj = config.LibvirtConfigGuestSMBIOS()
+
+        xml = obj.to_xml()
+        self.assertXmlEqual(xml, """
+            <smbios mode="sysinfo"/>
+        """)
+
+
+class LibvirtConfigGuestSysinfoTest(LibvirtConfigBaseTest):
+
+    def test_config_simple(self):
+        obj = config.LibvirtConfigGuestSysinfo()
+
+        xml = obj.to_xml()
+        self.assertXmlEqual(xml, """
+            <sysinfo type="smbios"/>
+        """)
+
+    def test_config_bios(self):
+        obj = config.LibvirtConfigGuestSysinfo()
+        obj.bios_vendor = "Acme"
+        obj.bios_version = "6.6.6"
+
+        xml = obj.to_xml()
+        self.assertXmlEqual(xml, """
+            <sysinfo type="smbios">
+              <bios>
+                <entry name="vendor">Acme</entry>
+                <entry name="version">6.6.6</entry>
+              </bios>
+            </sysinfo>
+        """)
+
+    def test_config_system(self):
+        obj = config.LibvirtConfigGuestSysinfo()
+        obj.system_manufacturer = "Acme"
+        obj.system_product = "Wile Coyote"
+        obj.system_version = "6.6.6"
+        obj.system_serial = "123456"
+        obj.system_uuid = "c7a5fdbd-edaf-9455-926a-d65c16db1809"
+
+        xml = obj.to_xml()
+        self.assertXmlEqual(xml, """
+            <sysinfo type="smbios">
+              <system>
+                <entry name="manufacturer">Acme</entry>
+                <entry name="product">Wile Coyote</entry>
+                <entry name="version">6.6.6</entry>
+                <entry name="serial">123456</entry>
+                <entry name="uuid">c7a5fdbd-edaf-9455-926a-d65c16db1809</entry>
+              </system>
+            </sysinfo>
+        """)
+
+    def test_config_mixed(self):
+        obj = config.LibvirtConfigGuestSysinfo()
+        obj.bios_vendor = "Acme"
+        obj.system_manufacturer = "Acme"
+        obj.system_product = "Wile Coyote"
+        obj.system_uuid = "c7a5fdbd-edaf-9455-926a-d65c16db1809"
+
+        xml = obj.to_xml()
+        self.assertXmlEqual(xml, """
+            <sysinfo type="smbios">
+              <bios>
+                <entry name="vendor">Acme</entry>
+              </bios>
+              <system>
+                <entry name="manufacturer">Acme</entry>
+                <entry name="product">Wile Coyote</entry>
+                <entry name="uuid">c7a5fdbd-edaf-9455-926a-d65c16db1809</entry>
+              </system>
+            </sysinfo>
+        """)
+
+
 class LibvirtConfigGuestDiskTest(LibvirtConfigBaseTest):
 
     def test_config_file(self):
@@ -366,7 +444,7 @@ class LibvirtConfigGuestDiskTest(LibvirtConfigBaseTest):
         self.assertXmlEqual(xml, """
             <disk type="network" device="disk">
               <driver name="qemu" type="qcow2"/>
-              <source protocol="iscsi" name="foo.bar.com"/>
+              <source name="foo.bar.com" protocol="iscsi"/>
               <target bus="ide" dev="/dev/hda"/>
             </disk>""")
 
@@ -387,7 +465,7 @@ class LibvirtConfigGuestDiskTest(LibvirtConfigBaseTest):
         self.assertXmlEqual(xml, """
             <disk type="network" device="disk">
               <driver name="qemu" type="raw"/>
-              <source protocol="rbd" name="pool/image"/>
+              <source name="pool/image" protocol="rbd"/>
               <auth username="foo">
                 <secret type="ceph"
                 uuid="b38a3f43-4be2-4046-897f-b67c2f5e0147"/>
@@ -395,6 +473,34 @@ class LibvirtConfigGuestDiskTest(LibvirtConfigBaseTest):
               <target bus="virtio" dev="/dev/vda"/>
             </disk>""")
 
+    def test_config_iotune(self):
+        obj = config.LibvirtConfigGuestDisk()
+        obj.source_type = "file"
+        obj.source_path = "/tmp/hello"
+        obj.target_dev = "/dev/hda"
+        obj.target_bus = "ide"
+        obj.disk_read_bytes_sec = 1024000
+        obj.disk_read_iops_sec = 1000
+        obj.disk_total_bytes_sec = 2048000
+        obj.disk_write_bytes_sec = 1024000
+        obj.disk_write_iops_sec = 1000
+        obj.disk_total_iops_sec = 2000
+
+        xml = obj.to_xml()
+        self.assertXmlEqual(xml, """
+            <disk type="file" device="disk">
+              <source file="/tmp/hello"/>
+              <target bus="ide" dev="/dev/hda"/>
+              <iotune>
+                <read_bytes_sec>1024000</read_bytes_sec>
+                <read_iops_sec>1000</read_iops_sec>
+                <write_bytes_sec>1024000</write_bytes_sec>
+                <write_iops_sec>1000</write_iops_sec>
+                <total_bytes_sec>2048000</total_bytes_sec>
+                <total_iops_sec>2000</total_iops_sec>
+              </iotune>
+            </disk>""")
+
 
 class LibvirtConfigGuestFilesysTest(LibvirtConfigBaseTest):
 
@@ -461,6 +567,29 @@ class LibvirtConfigGuestConsoleTest(LibvirtConfigBaseTest):
             <console type="pty"/>""")
 
 
+class LibvirtConfigGuestChannelTest(LibvirtConfigBaseTest):
+    def test_config_spice_minimal(self):
+        obj = config.LibvirtConfigGuestChannel()
+        obj.type = "spicevmc"
+
+        xml = obj.to_xml()
+        self.assertXmlEqual(xml, """
+            <channel type="spicevmc">
+              <target type='virtio'/>
+            </channel>""")
+
+    def test_config_spice_full(self):
+        obj = config.LibvirtConfigGuestChannel()
+        obj.type = "spicevmc"
+        obj.target_name = "com.redhat.spice.0"
+
+        xml = obj.to_xml()
+        self.assertXmlEqual(xml, """
+            <channel type="spicevmc">
+              <target type='virtio' name='com.redhat.spice.0'/>
+            </channel>""")
+
+
 class LibvirtConfigGuestInterfaceTest(LibvirtConfigBaseTest):
     def test_config_ethernet(self):
         obj = config.LibvirtConfigGuestInterface()
@@ -468,13 +597,25 @@ class LibvirtConfigGuestInterfaceTest(LibvirtConfigBaseTest):
         obj.mac_addr = "DE:AD:BE:EF:CA:FE"
         obj.model = "virtio"
         obj.target_dev = "vnet0"
+        obj.driver_name = "vhost"
+        obj.vif_inbound_average = 1024000
+        obj.vif_inbound_peak = 10240000
+        obj.vif_inbound_burst = 1024000
+        obj.vif_outbound_average = 1024000
+        obj.vif_outbound_peak = 10240000
+        obj.vif_outbound_burst = 1024000
 
         xml = obj.to_xml()
         self.assertXmlEqual(xml, """
             <interface type="ethernet">
               <mac address="DE:AD:BE:EF:CA:FE"/>
               <model type="virtio"/>
+              <driver name="vhost"/>
               <target dev="vnet0"/>
+              <bandwidth>
+                <inbound average="1024000" peak="10240000" burst="1024000"/>
+                <outbound average="1024000" peak="10240000" burst="1024000"/>
+              </bandwidth>
             </interface>""")
 
     def test_config_bridge(self):
@@ -483,8 +624,15 @@ class LibvirtConfigGuestInterfaceTest(LibvirtConfigBaseTest):
         obj.source_dev = "br0"
         obj.mac_addr = "DE:AD:BE:EF:CA:FE"
         obj.model = "virtio"
+        obj.target_dev = "tap12345678"
         obj.filtername = "clean-traffic"
         obj.filterparams.append({"key": "IP", "value": "192.168.122.1"})
+        obj.vif_inbound_average = 1024000
+        obj.vif_inbound_peak = 10240000
+        obj.vif_inbound_burst = 1024000
+        obj.vif_outbound_average = 1024000
+        obj.vif_outbound_peak = 10240000
+        obj.vif_outbound_burst = 1024000
 
         xml = obj.to_xml()
         self.assertXmlEqual(xml, """
@@ -492,9 +640,14 @@ class LibvirtConfigGuestInterfaceTest(LibvirtConfigBaseTest):
               <mac address="DE:AD:BE:EF:CA:FE"/>
               <model type="virtio"/>
               <source bridge="br0"/>
+              <target dev="tap12345678"/>
               <filterref filter="clean-traffic">
                 <parameter name="IP" value="192.168.122.1"/>
               </filterref>
+              <bandwidth>
+                <inbound average="1024000" peak="10240000" burst="1024000"/>
+                <outbound average="1024000" peak="10240000" burst="1024000"/>
+              </bandwidth>
             </interface>""")
 
     def test_config_bridge_ovs(self):
@@ -503,6 +656,7 @@ class LibvirtConfigGuestInterfaceTest(LibvirtConfigBaseTest):
         obj.source_dev = "br0"
         obj.mac_addr = "DE:AD:BE:EF:CA:FE"
         obj.model = "virtio"
+        obj.target_dev = "tap12345678"
         obj.vporttype = "openvswitch"
         obj.vportparams.append({"key": "instanceid", "value": "foobar"})
 
@@ -512,6 +666,7 @@ class LibvirtConfigGuestInterfaceTest(LibvirtConfigBaseTest):
               <mac address="DE:AD:BE:EF:CA:FE"/>
               <model type="virtio"/>
               <source bridge="br0"/>
+              <target dev="tap12345678"/>
               <virtualport type="openvswitch">
                 <parameters instanceid="foobar"/>
               </virtualport>
@@ -522,6 +677,7 @@ class LibvirtConfigGuestInterfaceTest(LibvirtConfigBaseTest):
         obj.net_type = "direct"
         obj.mac_addr = "DE:AD:BE:EF:CA:FE"
         obj.model = "virtio"
+        obj.target_dev = "tap12345678"
         obj.source_dev = "eth0"
         obj.vporttype = "802.1Qbh"
 
@@ -530,7 +686,8 @@ class LibvirtConfigGuestInterfaceTest(LibvirtConfigBaseTest):
             <interface type="direct">
               <mac address="DE:AD:BE:EF:CA:FE"/>
               <model type="virtio"/>
-              <source mode="private" dev="eth0"/>
+              <source dev="eth0" mode="private"/>
+              <target dev="tap12345678"/>
               <virtualport type="802.1Qbh"/>
             </interface>""")
 
@@ -667,13 +824,21 @@ class LibvirtConfigGuestTest(LibvirtConfigBaseTest):
         obj.virt_type = "kvm"
         obj.memory = 1024 * 1024 * 100
         obj.vcpus = 2
+        obj.cpu_shares = 100
+        obj.cpu_quota = 50000
+        obj.cpu_period = 25000
         obj.name = "demo"
         obj.uuid = "b38a3f43-4be2-4046-897f-b67c2f5e0147"
         obj.os_type = "linux"
         obj.os_boot_dev = "hd"
+        obj.os_smbios = config.LibvirtConfigGuestSMBIOS()
         obj.acpi = True
         obj.apic = True
 
+        obj.sysinfo = config.LibvirtConfigGuestSysinfo()
+        obj.sysinfo.bios_vendor = "Acme"
+        obj.sysinfo.system_version = "1.0.0"
+
         disk = config.LibvirtConfigGuestDisk()
         disk.source_type = "file"
         disk.source_path = "/tmp/img"
@@ -689,14 +854,28 @@ class LibvirtConfigGuestTest(LibvirtConfigBaseTest):
               <name>demo</name>
               <memory>104857600</memory>
               <vcpu>2</vcpu>
+              <sysinfo type='smbios'>
+                 <bios>
+                   <entry name="vendor">Acme</entry>
+                 </bios>
+                 <system>
+                   <entry name="version">1.0.0</entry>
+                 </system>
+              </sysinfo>
               <os>
                 <type>linux</type>
                 <boot dev="hd"/>
+                <smbios mode="sysinfo"/>
               </os>
               <features>
                 <acpi/>
                 <apic/>
               </features>
+              <cputune>
+                <shares>100</shares>
+                <quota>50000</quota>
+                <period>25000</period>
+              </cputune>
               <devices>
                 <disk type="file" device="disk">
                   <source file="/tmp/img"/>
