@@ -77,9 +77,11 @@ class S3APITestCase(test.TestCase):
                                calling_format=s3.OrdinaryCallingFormat())
         self.conn = conn
 
-        def get_http_connection(host, is_secure):
+        def get_http_connection(*args):
             """Get a new S3 connection, don't attempt to reuse connections."""
-            return self.conn.new_http_connection(host, is_secure)
+            # The calling function passed the parameters according to the
+            # the boto version.
+            return self.conn.new_http_connection(*args)
 
         self.conn.get_http_connection = get_http_connection
 
