@@ -22,12 +22,14 @@ Test suite for VMwareAPI.
 import urllib2
 
 import mox
+from oslo.config import cfg
 
 from nova.compute import power_state
 from nova.compute import task_states
 from nova import context
 from nova import db
 from nova import exception
+from nova.openstack.common import jsonutils
 from nova import test
 import nova.tests.image.fake
 from nova.tests import matchers
@@ -36,7 +38,9 @@ from nova.tests.vmwareapi import db_fakes
 from nova.tests.vmwareapi import stubs
 from nova.virt.vmwareapi import driver
 from nova.virt.vmwareapi import fake as vmwareapi_fake
+from nova.virt.vmwareapi import vim
 from nova.virt.vmwareapi import vm_util
+from nova.virt.vmwareapi import vmops
 from nova.virt.vmwareapi import volume_util
 
 
@@ -54,6 +58,47 @@ class fake_http_resp(object):
         return "console log"
 
 
+class VMwareAPIConfTestCase(test.TestCase):
+    """Unit tests for VMWare API configurations."""
+    def setUp(self):
+        super(VMwareAPIConfTestCase, self).setUp()
+
+    def tearDown(self):
+        super(VMwareAPIConfTestCase, self).tearDown()
+
+    def test_configure_without_wsdl_loc_override(self):
+        # Test the default configuration behavior. By default,
+        # use the WSDL sitting on the host we are talking to in
+        # order to bind the SOAP client.
+        wsdl_loc = cfg.CONF.vmwareapi_wsdl_loc
+        self.assertIsNone(wsdl_loc)
+        wsdl_url = vim.Vim.get_wsdl_url("https", "www.example.com")
+        url = vim.Vim.get_soap_url("https", "www.example.com")
+        self.assertEqual("https://www.example.com/sdk/vimService.wsdl",
+                         wsdl_url)
+        self.assertEqual("https://www.example.com/sdk", url)
+
+    def test_configure_with_wsdl_loc_override(self):
+        # Use the setting vmwareapi_wsdl_loc to override the
+        # default path to the WSDL.
+        #
+        # This is useful as a work-around for XML parsing issues
+        # found when using some WSDL in combination with some XML
+        # parsers.
+        #
+        # The wsdl_url should point to a different host than the one we
+        # are actually going to send commands to.
+        fake_wsdl = "https://www.test.com/sdk/foo.wsdl"
+        self.flags(vmwareapi_wsdl_loc=fake_wsdl)
+        wsdl_loc = cfg.CONF.vmwareapi_wsdl_loc
+        self.assertIsNotNone(wsdl_loc)
+        self.assertEqual(fake_wsdl, wsdl_loc)
+        wsdl_url = vim.Vim.get_wsdl_url("https", "www.example.com")
+        url = vim.Vim.get_soap_url("https", "www.example.com")
+        self.assertEqual(fake_wsdl, wsdl_url)
+        self.assertEqual("https://www.example.com/sdk", url)
+
+
 class VMwareAPIVMTestCase(test.TestCase):
     """Unit tests for Vmware API connection calls."""
 
@@ -65,8 +110,10 @@ class VMwareAPIVMTestCase(test.TestCase):
                    vmwareapi_host_password='test_pass',
                    vnc_enabled=False,
                    use_linked_clone=False)
+        self.flags(datastore_regex='.*', group='vmware')
         self.user_id = 'fake'
         self.project_id = 'fake'
+        self.node_name = 'test_url'
         self.context = context.RequestContext(self.user_id, self.project_id)
         vmwareapi_fake.reset()
         db_fakes.stub_out_db_instance_api(self.stubs)
@@ -182,7 +229,7 @@ class VMwareAPIVMTestCase(test.TestCase):
         info = self.conn.get_info({'uuid': 'fake-uuid'})
         self._check_vm_info(info, power_state.RUNNING)
 
-    def test_snapshot(self):
+    def _test_snapshot(self):
         expected_calls = [
             {'args': (),
              'kwargs':
@@ -201,6 +248,17 @@ class VMwareAPIVMTestCase(test.TestCase):
         self._check_vm_info(info, power_state.RUNNING)
         self.assertIsNone(func_call_matcher.match())
 
+    def test_snapshot(self):
+        # Ensure VMwareVCVMOps's _get_copy_virtual_disk_spec is getting called
+        self.mox.StubOutWithMock(vmops.VMwareVCVMOps,
+                                 '_get_copy_virtual_disk_spec')
+        self.conn._vmops._get_copy_virtual_disk_spec(
+                mox.IgnoreArg(),
+                mox.IgnoreArg(),
+                mox.IgnoreArg()).AndReturn(None)
+        self.mox.ReplayAll()
+        self._test_snapshot()
+
     def test_snapshot_non_existent(self):
         self._create_instance_in_the_db()
         self.assertRaises(exception.InstanceNotFound, self.conn.snapshot,
@@ -379,11 +437,13 @@ class VMwareAPIVMTestCase(test.TestCase):
                           self.instance)
 
     def test_get_vnc_console(self):
-        self._create_instance_in_the_db()
         self._create_vm()
+        fake_vm = vmwareapi_fake._get_objects("VirtualMachine")[0]
+        fake_vm_id = int(fake_vm.obj.value.replace('vm-', ''))
         vnc_dict = self.conn.get_vnc_console(self.instance)
         self.assertEquals(vnc_dict['host'], "ha-host")
-        self.assertEquals(vnc_dict['port'], 5910)
+        self.assertEquals(vnc_dict['port'], cfg.CONF.vnc_port +
+                          fake_vm_id % cfg.CONF.vnc_port_total)
 
     def test_host_ip_addr(self):
         self.assertEquals(self.conn.get_host_ip_addr(), "test_url")
@@ -447,3 +507,43 @@ class VMwareAPIHostTestCase(test.TestCase):
 
     def test_host_maintenance_off(self):
         self._test_host_action(self.conn.host_maintenance_mode, False)
+
+    def test_invalid_datastore_regex(self):
+        # Tests if we raise an exception for Invalid Regular Expression in
+        # vmware_datastore_regex
+        self.flags(vmwareapi_host_ip='test_url',
+                   vmwareapi_host_username='test_username',
+                   vmwareapi_host_password='test_pass',
+                   vnc_enabled=False,
+                   use_linked_clone=False)
+        self.flags(datastore_regex='fake-ds(01', group='vmware')
+        self.assertRaises(exception.InvalidInput, driver.VMwareVCDriver, None)
+
+
+class VMwareAPIVCDriverTestCase(VMwareAPIVMTestCase):
+
+    def setUp(self):
+        super(VMwareAPIVCDriverTestCase, self).setUp()
+        self.flags(vmwareapi_cluster_name='test_cluster')
+        self.flags(vnc_enabled=False)
+        self.conn = driver.VMwareVCDriver(None, False)
+
+    def tearDown(self):
+        super(VMwareAPIVCDriverTestCase, self).tearDown()
+        vmwareapi_fake.cleanup()
+
+    def test_get_available_resource(self):
+        stats = self.conn.get_available_resource(self.node_name)
+        cpu_info = {"model": ["Intel(R) Xeon(R)", "Intel(R) Xeon(R)"],
+                    "vendor": ["Intel", "Intel"],
+                    "topology": {"cores": 16,
+                                 "threads": 32}}
+        self.assertEquals(stats['vcpus'], 32)
+        self.assertEquals(stats['local_gb'], 1024)
+        self.assertEquals(stats['local_gb_used'], 1024 - 500)
+        self.assertEquals(stats['memory_mb'], 1000)
+        self.assertEquals(stats['memory_mb_used'], 500)
+        self.assertEquals(stats['hypervisor_type'], 'VMware vCenter Server')
+        self.assertEquals(stats['hypervisor_version'], '5.1.0')
+        self.assertEquals(stats['hypervisor_hostname'], self.node_name)
+        self.assertEquals(stats['cpu_info'], jsonutils.dumps(cpu_info))
