@@ -32,15 +32,15 @@ import tempfile
 if os.name != 'nt':
     import crypt
 
+from oslo.config import cfg
+
 from nova import exception
-from nova import flags
-from nova.openstack.common import cfg
 from nova.openstack.common import jsonutils
 from nova.openstack.common import log as logging
+from nova import paths
 from nova import utils
-from nova.virt.disk import guestfs
-from nova.virt.disk import loop
-from nova.virt.disk import nbd
+from nova.virt.disk.mount import api as mount
+from nova.virt.disk.vfs import api as vfs
 from nova.virt import images
 
 
@@ -48,11 +48,8 @@ LOG = logging.getLogger(__name__)
 
 disk_opts = [
     cfg.StrOpt('injected_network_template',
-               default='$pybasedir/nova/virt/interfaces.template',
+               default=paths.basedir_def('nova/virt/interfaces.template'),
                help='Template file for injected network'),
-    cfg.ListOpt('img_handlers',
-                default=['loop', 'nbd', 'guestfs'],
-                help='Order of methods used to mount disk images'),
 
     # NOTE(yamahata): ListOpt won't work because the command may include a
     #                 comma. For example:
@@ -76,14 +73,14 @@ disk_opts = [
                          'The format is <os_type>=<mkfs command>'),
     ]
 
-FLAGS = flags.FLAGS
-FLAGS.register_opts(disk_opts)
+CONF = cfg.CONF
+CONF.register_opts(disk_opts)
 
 _MKFS_COMMAND = {}
 _DEFAULT_MKFS_COMMAND = None
 
 
-for s in FLAGS.virt_mkfs:
+for s in CONF.virt_mkfs:
     # NOTE(yamahata): mkfs command may includes '=' for its options.
     #                 So item.partition('=') doesn't work here
     os_type, mkfs_command = s.split('=', 1)
@@ -97,12 +94,16 @@ def mkfs(os_type, fs_label, target):
     mkfs_command = (_MKFS_COMMAND.get(os_type, _DEFAULT_MKFS_COMMAND) or
                     '') % locals()
     if mkfs_command:
-        utils.execute(*mkfs_command.split())
+        utils.execute(*mkfs_command.split(), run_as_root=True)
 
 
-def resize2fs(image, check_exit_code=False):
-    utils.execute('e2fsck', '-fp', image, check_exit_code=check_exit_code)
-    utils.execute('resize2fs', image, check_exit_code=check_exit_code)
+def resize2fs(image, check_exit_code=False, run_as_root=False):
+    utils.execute('e2fsck', '-fp', image,
+                  check_exit_code=check_exit_code,
+                  run_as_root=run_as_root)
+    utils.execute('resize2fs', image,
+                  check_exit_code=check_exit_code,
+                  run_as_root=run_as_root)
 
 
 def get_disk_size(path):
@@ -112,13 +113,11 @@ def get_disk_size(path):
     :returns: Size (in bytes) of the given disk image as it would be seen
               by a virtual machine.
     """
-    size = images.qemu_img_info(path)['virtual size']
-    size = size.split('(')[1].split()[0]
-    return int(size)
+    return images.qemu_img_info(path).virtual_size
 
 
 def extend(image, size):
-    """Increase image to size"""
+    """Increase image to size."""
     virt_size = get_disk_size(image)
     if virt_size >= size:
         return
@@ -130,48 +129,42 @@ def extend(image, size):
 def can_resize_fs(image, size, use_cow=False):
     """Check whether we can resize contained file system."""
 
+    LOG.debug(_('Checking if we can resize image %(image)s. '
+                'size=%(size)s, CoW=%(use_cow)s'), locals())
+
     # Check that we're increasing the size
     virt_size = get_disk_size(image)
     if virt_size >= size:
+        LOG.debug(_('Cannot resize filesystem %s to a smaller size.'),
+                  image)
         return False
 
     # Check the image is unpartitioned
     if use_cow:
-        # Try to mount an unpartitioned qcow2 image
         try:
-            inject_data(image, use_cow=True)
-        except exception.NovaException:
+            fs = vfs.VFS.instance_for_image(image, 'qcow2', None)
+            fs.setup()
+            fs.teardown()
+        except exception.NovaException, e:
+            LOG.debug(_('Unable to mount image %(image)s with '
+                        'error %(error)s. Cannot resize.'),
+                      {'image': image,
+                       'error': e})
             return False
     else:
         # For raw, we can directly inspect the file system
         try:
             utils.execute('e2label', image)
-        except exception.ProcessExecutionError:
+        except exception.ProcessExecutionError, e:
+            LOG.debug(_('Unable to determine label for image %(image)s with '
+                        'error %(errror)s. Cannot resize.'),
+                      {'image': image,
+                       'error': e})
             return False
 
     return True
 
 
-def bind(src, target, instance_name):
-    """Bind device to a filesytem"""
-    if src:
-        utils.execute('touch', target, run_as_root=True)
-        utils.execute('mount', '-o', 'bind', src, target,
-                run_as_root=True)
-        s = os.stat(src)
-        cgroup_info = "b %s:%s rwm\n" % (os.major(s.st_rdev),
-                                         os.minor(s.st_rdev))
-        cgroups_path = ("/sys/fs/cgroup/devices/libvirt/lxc/"
-                        "%s/devices.allow" % instance_name)
-        utils.execute('tee', cgroups_path,
-                      process_input=cgroup_info, run_as_root=True)
-
-
-def unbind(target):
-    if target:
-        utils.execute('umount', target, run_as_root=True)
-
-
 class _DiskImage(object):
     """Provide operations on a disk image file."""
 
@@ -182,25 +175,14 @@ class _DiskImage(object):
         self.image = image
         self.partition = partition
         self.mount_dir = mount_dir
+        self.use_cow = use_cow
 
         # Internal
         self._mkdir = False
         self._mounter = None
         self._errors = []
 
-        # As a performance tweak, don't bother trying to
-        # directly loopback mount a cow image.
-        self.handlers = FLAGS.img_handlers[:]
-        if use_cow and 'loop' in self.handlers:
-            self.handlers.remove('loop')
-
-        if not self.handlers:
-            msg = _('no capable image handler configured')
-            raise exception.NovaException(msg)
-
         if mount_dir:
-            # Note the os.path.ismount() shortcut doesn't
-            # work with libguestfs due to permissions issues.
             device = self._device_for_path(mount_dir)
             if device:
                 self._reset(device)
@@ -208,22 +190,21 @@ class _DiskImage(object):
     @staticmethod
     def _device_for_path(path):
         device = None
+        path = os.path.realpath(path)
         with open("/proc/mounts", 'r') as ifp:
             for line in ifp:
                 fields = line.split()
-                if fields[1] == os.path.realpath(path):
+                if fields[1] == path:
                     device = fields[0]
                     break
         return device
 
     def _reset(self, device):
         """Reset internal state for a previously mounted directory."""
-        mounter_cls = self._handler_class(device=device)
-        mounter = mounter_cls(image=self.image,
-                              partition=self.partition,
-                              mount_dir=self.mount_dir,
-                              device=device)
-        self._mounter = mounter
+        self._mounter = mount.Mount.instance_for_device(self.image,
+                                                        self.mount_dir,
+                                                        self.partition,
+                                                        device)
 
         mount_name = os.path.basename(self.mount_dir or '')
         self._mkdir = mount_name.startswith(self.tmp_prefix)
@@ -233,17 +214,6 @@ class _DiskImage(object):
         """Return the collated errors from all operations."""
         return '\n--\n'.join([''] + self._errors)
 
-    @staticmethod
-    def _handler_class(mode=None, device=None):
-        """Look up the appropriate class to use based on MODE or DEVICE."""
-        for cls in (loop.Mount, nbd.Mount, guestfs.Mount):
-            if mode and cls.mode == mode:
-                return cls
-            elif device and cls.device_id_string in device:
-                return cls
-        msg = _("no disk image handler for: %s") % mode or device
-        raise exception.NovaException(msg)
-
     def mount(self):
         """Mount a disk image, using the object attributes.
 
@@ -259,29 +229,34 @@ class _DiskImage(object):
             self.mount_dir = tempfile.mkdtemp(prefix=self.tmp_prefix)
             self._mkdir = True
 
-        try:
-            for h in self.handlers:
-                mounter_cls = self._handler_class(h)
-                mounter = mounter_cls(image=self.image,
-                                      partition=self.partition,
-                                      mount_dir=self.mount_dir)
-                if mounter.do_mount():
-                    self._mounter = mounter
-                    break
-                else:
-                    LOG.debug(mounter.error)
-                    self._errors.append(mounter.error)
-        finally:
-            if not self._mounter:
-                self.umount()  # rmdir
+        imgfmt = "raw"
+        if self.use_cow:
+            imgfmt = "qcow2"
+
+        mounter = mount.Mount.instance_for_format(self.image,
+                                                  self.mount_dir,
+                                                  self.partition,
+                                                  imgfmt)
+        if mounter.do_mount():
+            self._mounter = mounter
+        else:
+            LOG.debug(mounter.error)
+            self._errors.append(mounter.error)
 
         return bool(self._mounter)
 
     def umount(self):
-        """Unmount a disk image from the file system."""
+        """Umount a mount point from the filesystem."""
+        if self._mounter:
+            self._mounter.do_umount()
+            self._mounter = None
+
+    def teardown(self):
+        """Remove a disk image from the file system."""
         try:
             if self._mounter:
-                self._mounter.do_umount()
+                self._mounter.do_teardown()
+                self._mounter = None
         finally:
             if self._mkdir:
                 os.rmdir(self.mount_dir)
@@ -289,26 +264,47 @@ class _DiskImage(object):
 
 # Public module functions
 
-def inject_data(image,
-                key=None, net=None, metadata=None, admin_password=None,
-                files=None, partition=None, use_cow=False):
-    """Injects a ssh key and optionally net data into a disk image.
+def inject_data(image, key=None, net=None, metadata=None, admin_password=None,
+                files=None, partition=None, use_cow=False, mandatory=()):
+    """Inject the specified items into a disk image.
+
+    If an item name is not specified in the MANDATORY iterable, then a warning
+    is logged on failure to inject that item, rather than raising an exception.
 
     it will mount the image as a fully partitioned disk and attempt to inject
     into the specified partition number.
 
-    If partition is not specified it mounts the image as a single partition.
+    If PARTITION is not specified the image is mounted as a single partition.
 
+    Returns True if all requested operations completed without issue.
+    Raises an exception if a mandatory item can't be injected.
     """
-    img = _DiskImage(image=image, partition=partition, use_cow=use_cow)
-    if img.mount():
-        try:
-            inject_data_into_fs(img.mount_dir,
-                                key, net, metadata, admin_password, files)
-        finally:
-            img.umount()
-    else:
-        raise exception.NovaException(img.errors)
+    LOG.debug(_("Inject data image=%(image)s key=%(key)s net=%(net)s "
+                "metadata=%(metadata)s admin_password=ha-ha-not-telling-you "
+                "files=%(files)s partition=%(partition)s use_cow=%(use_cow)s")
+              % locals())
+    fmt = "raw"
+    if use_cow:
+        fmt = "qcow2"
+    try:
+        fs = vfs.VFS.instance_for_image(image, fmt, partition)
+        fs.setup()
+    except Exception as e:
+        # If a mandatory item is passed to this function,
+        # then reraise the exception to indicate the error.
+        for inject in mandatory:
+            inject_val = locals()[inject]
+            if inject_val:
+                raise
+        LOG.warn(_('Ignoring error injecting data into image '
+                   '(%(e)s)') % locals())
+        return False
+
+    try:
+        return inject_data_into_fs(fs, key, net, metadata,
+                                   admin_password, files, mandatory)
+    finally:
+        fs.teardown()
 
 
 def setup_container(image, container_dir, use_cow=False):
@@ -326,89 +322,91 @@ def setup_container(image, container_dir, use_cow=False):
         raise exception.NovaException(img.errors)
 
 
-def destroy_container(container_dir):
-    """Destroy the container once it terminates.
+def teardown_container(container_dir):
+    """Teardown the container rootfs mounting once it is spawned.
 
     It will umount the container that is mounted,
-    and delete any  linked devices.
+    and delete any linked devices.
+    """
+    try:
+        img = _DiskImage(image=None, mount_dir=container_dir)
+        img.teardown()
+    except Exception, exn:
+        LOG.exception(_('Failed to teardown ntainer filesystem: %s'), exn)
+
+
+def clean_lxc_namespace(container_dir):
+    """Clean up the container namespace rootfs mounting one spawned.
+
+    It will umount the mounted names that is mounted
+    but leave the linked deivces alone.
     """
     try:
         img = _DiskImage(image=None, mount_dir=container_dir)
         img.umount()
     except Exception, exn:
-        LOG.exception(_('Failed to unmount container filesystem: %s'), exn)
+        LOG.exception(_('Failed to umount container filesystem: %s'), exn)
 
 
-def inject_data_into_fs(fs, key, net, metadata, admin_password, files):
+def inject_data_into_fs(fs, key, net, metadata, admin_password, files,
+                        mandatory=()):
     """Injects data into a filesystem already mounted by the caller.
     Virt connections can call this directly if they mount their fs
-    in a different way to inject_data
+    in a different way to inject_data.
+
+    If an item name is not specified in the MANDATORY iterable, then a warning
+    is logged on failure to inject that item, rather than raising an exception.
+
+    Returns True if all requested operations completed without issue.
+    Raises an exception if a mandatory item can't be injected.
     """
-    if key:
-        _inject_key_into_fs(key, fs)
-    if net:
-        _inject_net_into_fs(net, fs)
-    if metadata:
-        _inject_metadata_into_fs(metadata, fs)
-    if admin_password:
-        _inject_admin_password_into_fs(admin_password, fs)
-    if files:
-        for (path, contents) in files:
-            _inject_file_into_fs(fs, path, contents)
+    status = True
+    for inject in ('key', 'net', 'metadata', 'admin_password', 'files'):
+        inject_val = locals()[inject]
+        inject_func = globals()['_inject_%s_into_fs' % inject]
+        if inject_val:
+            try:
+                inject_func(inject_val, fs)
+            except Exception as e:
+                if inject in mandatory:
+                    raise
+                LOG.warn(_('Ignoring error injecting %(inject)s into image '
+                           '(%(e)s)') % locals())
+                status = False
+    return status
 
 
-def _join_and_check_path_within_fs(fs, *args):
-    '''os.path.join() with safety check for injected file paths.
-
-    Join the supplied path components and make sure that the
-    resulting path we are injecting into is within the
-    mounted guest fs.  Trying to be clever and specifying a
-    path with '..' in it will hit this safeguard.
-    '''
-    absolute_path, _err = utils.execute('readlink', '-nm',
-                                        os.path.join(fs, *args),
-                                        run_as_root=True)
-    if not absolute_path.startswith(os.path.realpath(fs) + '/'):
-        raise exception.Invalid(_('injected file path not valid'))
-    return absolute_path
+def _inject_files_into_fs(files, fs):
+    for (path, contents) in files:
+        _inject_file_into_fs(fs, path, contents)
 
 
 def _inject_file_into_fs(fs, path, contents, append=False):
-    absolute_path = _join_and_check_path_within_fs(fs, path.lstrip('/'))
-
-    parent_dir = os.path.dirname(absolute_path)
-    utils.execute('mkdir', '-p', parent_dir, run_as_root=True)
-
-    args = []
+    LOG.debug(_("Inject file fs=%(fs)s path=%(path)s append=%(append)s") %
+              locals())
     if append:
-        args.append('-a')
-    args.append(absolute_path)
-
-    kwargs = dict(process_input=contents, run_as_root=True)
-
-    utils.execute('tee', *args, **kwargs)
+        fs.append_file(path, contents)
+    else:
+        fs.replace_file(path, contents)
 
 
 def _inject_metadata_into_fs(metadata, fs):
+    LOG.debug(_("Inject metadata fs=%(fs)s metadata=%(metadata)s") %
+              locals())
     metadata = dict([(m['key'], m['value']) for m in metadata])
     _inject_file_into_fs(fs, 'meta.js', jsonutils.dumps(metadata))
 
 
-def _setup_selinux_for_keys(fs):
+def _setup_selinux_for_keys(fs, sshdir):
     """Get selinux guests to ensure correct context on injected keys."""
 
-    se_cfg = _join_and_check_path_within_fs(fs, 'etc', 'selinux')
-    se_cfg, _err = utils.trycmd('readlink', '-e', se_cfg, run_as_root=True)
-    if not se_cfg:
+    if not fs.has_file(os.path.join("etc", "selinux")):
         return
 
-    rclocal = _join_and_check_path_within_fs(fs, 'etc', 'rc.local')
+    rclocal = os.path.join('etc', 'rc.local')
+    rc_d = os.path.join('etc', 'rc.d')
 
-    # Support systemd based systems
-    rc_d = _join_and_check_path_within_fs(fs, 'etc', 'rc.d')
-    rclocal_e, _err = utils.trycmd('readlink', '-e', rclocal, run_as_root=True)
-    rc_d_e, _err = utils.trycmd('readlink', '-e', rc_d, run_as_root=True)
-    if not rclocal_e and rc_d_e:
+    if not fs.has_file(rclocal) and fs.has_file(rc_d):
         rclocal = os.path.join(rc_d, 'rc.local')
 
     # Note some systems end rc.local with "exit 0"
@@ -417,15 +415,14 @@ def _setup_selinux_for_keys(fs):
     restorecon = [
         '\n',
         '# Added by Nova to ensure injected ssh keys have the right context\n',
-        'restorecon -RF /root/.ssh/ 2>/dev/null || :\n',
+        'restorecon -RF %s 2>/dev/null || :\n' % sshdir,
     ]
 
     if not fs.has_file(rclocal):
         restorecon.insert(0, '#!/bin/sh')
 
-    rclocal_rel = os.path.relpath(rclocal, fs)
-    _inject_file_into_fs(fs, rclocal_rel, ''.join(restorecon), append=True)
-    utils.execute('chmod', 'a+x', rclocal, run_as_root=True)
+    _inject_file_into_fs(fs, rclocal, ''.join(restorecon), append=True)
+    fs.set_permissions(rclocal, 0700)
 
 
 def _inject_key_into_fs(key, fs):
@@ -434,12 +431,15 @@ def _inject_key_into_fs(key, fs):
     key is an ssh key string.
     fs is the path to the base of the filesystem into which to inject the key.
     """
-    sshdir = _join_and_check_path_within_fs(fs, 'root', '.ssh')
-    utils.execute('mkdir', '-p', sshdir, run_as_root=True)
-    utils.execute('chown', 'root', sshdir, run_as_root=True)
-    utils.execute('chmod', '700', sshdir, run_as_root=True)
 
-    keyfile = os.path.join('root', '.ssh', 'authorized_keys')
+    LOG.debug(_("Inject key fs=%(fs)s key=%(key)s") %
+              locals())
+    sshdir = os.path.join('root', '.ssh')
+    fs.make_path(sshdir)
+    fs.set_ownership(sshdir, "root", "root")
+    fs.set_permissions(sshdir, 0700)
+
+    keyfile = os.path.join(sshdir, 'authorized_keys')
 
     key_data = ''.join([
         '\n',
@@ -450,8 +450,9 @@ def _inject_key_into_fs(key, fs):
     ])
 
     _inject_file_into_fs(fs, keyfile, key_data, append=True)
+    fs.set_permissions(keyfile, 0600)
 
-    _setup_selinux_for_keys(fs)
+    _setup_selinux_for_keys(fs, sshdir)
 
 
 def _inject_net_into_fs(net, fs):
@@ -459,10 +460,13 @@ def _inject_net_into_fs(net, fs):
 
     net is the contents of /etc/network/interfaces.
     """
-    netdir = _join_and_check_path_within_fs(fs, 'etc', 'network')
-    utils.execute('mkdir', '-p', netdir, run_as_root=True)
-    utils.execute('chown', 'root:root', netdir, run_as_root=True)
-    utils.execute('chmod', 755, netdir, run_as_root=True)
+
+    LOG.debug(_("Inject key fs=%(fs)s net=%(net)s") %
+              locals())
+    netdir = os.path.join('etc', 'network')
+    fs.make_path(netdir)
+    fs.set_ownership(netdir, "root", "root")
+    fs.set_permissions(netdir, 0744)
 
     netfile = os.path.join('etc', 'network', 'interfaces')
     _inject_file_into_fs(fs, netfile, net)
@@ -483,6 +487,9 @@ def _inject_admin_password_into_fs(admin_passwd, fs):
     # files from the instance filesystem to local files, make any
     # necessary changes, and then copy them back.
 
+    LOG.debug(_("Inject admin password fs=%(fs)s "
+                "admin_passwd=ha-ha-not-telling-you") %
+              locals())
     admin_user = 'root'
 
     fd, tmp_passwd = tempfile.mkstemp()
@@ -490,19 +497,27 @@ def _inject_admin_password_into_fs(admin_passwd, fs):
     fd, tmp_shadow = tempfile.mkstemp()
     os.close(fd)
 
-    passwd_path = _join_and_check_path_within_fs(fs, 'etc', 'passwd')
-    shadow_path = _join_and_check_path_within_fs(fs, 'etc', 'shadow')
+    passwd_path = os.path.join('etc', 'passwd')
+    shadow_path = os.path.join('etc', 'shadow')
 
-    utils.execute('cp', passwd_path, tmp_passwd, run_as_root=True)
-    utils.execute('cp', shadow_path, tmp_shadow, run_as_root=True)
-    _set_passwd(admin_user, admin_passwd, tmp_passwd, tmp_shadow)
-    utils.execute('cp', tmp_passwd, passwd_path, run_as_root=True)
-    os.unlink(tmp_passwd)
-    utils.execute('cp', tmp_shadow, shadow_path, run_as_root=True)
-    os.unlink(tmp_shadow)
+    passwd_data = fs.read_file(passwd_path)
+    shadow_data = fs.read_file(shadow_path)
+
+    new_shadow_data = _set_passwd(admin_user, admin_passwd,
+                                  passwd_data, shadow_data)
+
+    fs.replace_file(shadow_path, new_shadow_data)
 
 
-def _set_passwd(username, admin_passwd, passwd_file, shadow_file):
+def _generate_salt():
+    salt_set = ('abcdefghijklmnopqrstuvwxyz'
+                'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+                '0123456789./')
+    salt = 16 * ' '
+    return ''.join([random.choice(salt_set) for c in salt])
+
+
+def _set_passwd(username, admin_passwd, passwd_data, shadow_data):
     """set the password for username to admin_passwd
 
     The passwd_file is not modified.  The shadow_file is updated.
@@ -519,14 +534,10 @@ def _set_passwd(username, admin_passwd, passwd_file, shadow_file):
     if os.name == 'nt':
         raise exception.NovaException(_('Not implemented on Windows'))
 
-    salt_set = ('abcdefghijklmnopqrstuvwxyz'
-                'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
-                '0123456789./')
     # encryption algo - id pairs for crypt()
     algos = {'SHA-512': '$6$', 'SHA-256': '$5$', 'MD5': '$1$', 'DES': ''}
 
-    salt = 16 * ' '
-    salt = ''.join([random.choice(salt_set) for c in salt])
+    salt = _generate_salt()
 
     # crypt() depends on the underlying libc, and may not support all
     # forms of hash. We try md5 first. If we get only 13 characters back,
@@ -539,39 +550,34 @@ def _set_passwd(username, admin_passwd, passwd_file, shadow_file):
     if len(encrypted_passwd) == 13:
         encrypted_passwd = crypt.crypt(admin_passwd, algos['DES'] + salt)
 
-    try:
-        p_file = open(passwd_file, 'rb')
-        s_file = open(shadow_file, 'rb')
+    p_file = passwd_data.split("\n")
+    s_file = shadow_data.split("\n")
 
-        # username MUST exist in passwd file or it's an error
-        found = False
-        for entry in p_file:
-            split_entry = entry.split(':')
-            if split_entry[0] == username:
-                found = True
-                break
-        if not found:
-            msg = _('User %(username)s not found in password file.')
-            raise exception.NovaException(msg % username)
+     # username MUST exist in passwd file or it's an error
+    found = False
+    for entry in p_file:
+        split_entry = entry.split(':')
+        if split_entry[0] == username:
+            found = True
+            break
+    if not found:
+        msg = _('User %(username)s not found in password file.')
+        raise exception.NovaException(msg % username)
 
-        # update password in the shadow file.It's an error if the
-        # the user doesn't exist.
-        new_shadow = list()
-        found = False
-        for entry in s_file:
-            split_entry = entry.split(':')
-            if split_entry[0] == username:
-                split_entry[1] = encrypted_passwd
-                found = True
-            new_entry = ':'.join(split_entry)
-            new_shadow.append(new_entry)
-        s_file.close()
-        if not found:
-            msg = _('User %(username)s not found in shadow file.')
-            raise exception.NovaException(msg % username)
-        s_file = open(shadow_file, 'wb')
-        for entry in new_shadow:
-            s_file.write(entry)
-    finally:
-        p_file.close()
-        s_file.close()
+    # update password in the shadow file.It's an error if the
+    # the user doesn't exist.
+    new_shadow = list()
+    found = False
+    for entry in s_file:
+        split_entry = entry.split(':')
+        if split_entry[0] == username:
+            split_entry[1] = encrypted_passwd
+            found = True
+        new_entry = ':'.join(split_entry)
+        new_shadow.append(new_entry)
+
+    if not found:
+        msg = _('User %(username)s not found in shadow file.')
+        raise exception.NovaException(msg % username)
+
+    return "\n".join(new_shadow)
