@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2011 Red Hat, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,20 +18,13 @@
 import os
 import time
 
-from oslo_log import log as logging
-from oslo_service import loopingcall
-from oslo_utils import importutils
-
-from nova import exception
-from nova.i18n import _
-import nova.privsep.fs
-from nova.virt.image import model as imgmodel
+from nova.openstack.common import importutils
+from nova.openstack.common import log as logging
+from nova import utils
 
 LOG = logging.getLogger(__name__)
 
 MAX_DEVICE_WAIT = 30
-MAX_FILE_CHECKS = 6
-FILE_CHECK_INTERVAL = 0.25
 
 
 class Mount(object):
@@ -42,85 +37,38 @@ class Mount(object):
     mode = None  # to be overridden in subclasses
 
     @staticmethod
-    def instance_for_format(image, mountdir, partition):
-        """Get a Mount instance for the image type
-
-        :param image: instance of nova.virt.image.model.Image
-        :param mountdir: path to mount the image at
-        :param partition: partition number to mount
-        """
-        LOG.debug("Instance for format image=%(image)s "
-                  "mountdir=%(mountdir)s partition=%(partition)s",
-                  {'image': image, 'mountdir': mountdir,
-                   'partition': partition})
-
-        if isinstance(image, imgmodel.LocalFileImage):
-            if image.format == imgmodel.FORMAT_RAW:
-                LOG.debug("Using LoopMount")
-                return importutils.import_object(
-                    "nova.virt.disk.mount.loop.LoopMount",
-                    image, mountdir, partition)
-            else:
-                LOG.debug("Using NbdMount")
-                return importutils.import_object(
-                    "nova.virt.disk.mount.nbd.NbdMount",
-                    image, mountdir, partition)
-        elif isinstance(image, imgmodel.LocalBlockImage):
-            LOG.debug("Using BlockMount")
-            return importutils.import_object(
-                    "nova.virt.disk.mount.block.BlockMount",
-                    image, mountdir, partition)
-        else:
-            # TODO(berrange) We could mount RBDImage directly
-            # using kernel RBD block dev support.
-            #
-            # This is left as an enhancement for future
-            # motivated developers todo, since raising
-            # an exception is on par with what this
-            # code did historically
-            raise exception.UnsupportedImageModel(
-                image.__class__.__name__)
-
-    @staticmethod
-    def instance_for_device(image, mountdir, partition, device):
-        """Get a Mount instance for the device type
-
-        :param image: instance of nova.virt.image.model.Image
-        :param mountdir: path to mount the image at
-        :param partition: partition number to mount
-        :param device: mounted device path
-        """
-
-        LOG.debug("Instance for device image=%(image)s "
-                  "mountdir=%(mountdir)s partition=%(partition)s "
-                  "device=%(device)s",
-                  {'image': image, 'mountdir': mountdir,
-                   'partition': partition, 'device': device})
-
-        if "loop" in device:
-            LOG.debug("Using LoopMount")
+    def instance_for_format(imgfile, mountdir, partition, imgfmt):
+        LOG.debug(_("Instance for format imgfile=%(imgfile)s "
+                    "mountdir=%(mountdir)s partition=%(partition)s "
+                    "imgfmt=%(imgfmt)s") % locals())
+        if imgfmt == "raw":
+            LOG.debug(_("Using LoopMount"))
             return importutils.import_object(
                 "nova.virt.disk.mount.loop.LoopMount",
-                image, mountdir, partition, device)
-        elif "nbd" in device:
-            LOG.debug("Using NbdMount")
+                imgfile, mountdir, partition)
+        else:
+            LOG.debug(_("Using NbdMount"))
             return importutils.import_object(
                 "nova.virt.disk.mount.nbd.NbdMount",
-                image, mountdir, partition, device)
-        else:
-            LOG.debug("Using BlockMount")
+                imgfile, mountdir, partition)
+
+    @staticmethod
+    def instance_for_device(imgfile, mountdir, partition, device):
+        LOG.debug(_("Instance for device imgfile=%(imgfile)s "
+                    "mountdir=%(mountdir)s partition=%(partition)s "
+                    "device=%(device)s") % locals())
+        if "loop" in device:
+            LOG.debug(_("Using LoopMount"))
             return importutils.import_object(
-                "nova.virt.disk.mount.block.BlockMount",
-                image, mountdir, partition, device)
+                "nova.virt.disk.mount.loop.LoopMount",
+                imgfile, mountdir, partition, device)
+        else:
+            LOG.debug(_("Using NbdMount"))
+            return importutils.import_object(
+                "nova.virt.disk.mount.nbd.NbdMount",
+                imgfile, mountdir, partition, device)
 
     def __init__(self, image, mount_dir, partition=None, device=None):
-        """Create a new Mount instance
-
-        :param image: instance of nova.virt.image.model.Image
-        :param mount_dir: path to mount the image at
-        :param partition: partition number to mount
-        :param device: mounted device path
-        """
 
         # Input
         self.image = image
@@ -148,9 +96,8 @@ class Mount(object):
         if os.path.isabs(device) and os.path.exists(device):
             if device.startswith('/dev/mapper/'):
                 device = os.path.basename(device)
-                if 'p' in device:
-                    device, self.partition = device.rsplit('p', 1)
-                    self.device = os.path.join('/dev', device)
+                device, self.partition = device.rsplit('p', 1)
+                self.device = os.path.join('/dev', device)
 
     def get_dev(self):
         """Make the image available as a block device in the file system."""
@@ -167,10 +114,10 @@ class Mount(object):
         start_time = time.time()
         device = self._inner_get_dev()
         while not device:
-            LOG.info('Device allocation failed. Will retry in 2 seconds.')
+            LOG.info(_('Device allocation failed. Will retry in 2 seconds.'))
             time.sleep(2)
             if time.time() - start_time > MAX_DEVICE_WAIT:
-                LOG.warning('Device allocation failed after repeated retries.')
+                LOG.warn(_('Device allocation failed after repeated retries.'))
                 return False
             device = self._inner_get_dev()
         return True
@@ -185,7 +132,7 @@ class Mount(object):
     def map_dev(self):
         """Map partitions of the device to the file system namespace."""
         assert(os.path.exists(self.device))
-        LOG.debug("Map dev %s", self.device)
+        LOG.debug(_("Map dev %s"), self.device)
         automapped_path = '/dev/%sp%s' % (os.path.basename(self.device),
                                               self.partition)
 
@@ -199,26 +146,18 @@ class Mount(object):
             # Note kpartx can output warnings to stderr and succeed
             # Also it can output failures to stderr and "succeed"
             # So we just go on the existence of the mapped device
-            _out, err = nova.privsep.fs.create_device_maps(self.device)
-
-            @loopingcall.RetryDecorator(
-                    max_retry_count=MAX_FILE_CHECKS - 1,
-                    max_sleep_time=FILE_CHECK_INTERVAL,
-                    exceptions=IOError)
-            def recheck_path(map_path):
-                if not os.path.exists(map_path):
-                    raise IOError()
+            _out, err = utils.trycmd('kpartx', '-a', self.device,
+                                     run_as_root=True, discard_warnings=True)
 
             # Note kpartx does nothing when presented with a raw image,
             # so given we only use it when we expect a partitioned image, fail
-            try:
-                recheck_path(map_path)
-                self.mapped_device = map_path
-                self.mapped = True
-            except IOError:
+            if not os.path.exists(map_path):
                 if not err:
                     err = _('partition %s not found') % self.partition
                 self.error = _('Failed to map partitions: %s') % err
+            else:
+                self.mapped_device = map_path
+                self.mapped = True
         elif self.partition and os.path.exists(automapped_path):
             # Note auto mapping can be enabled with the 'max_part' option
             # to the nbd or loop kernel modules. Beware of possible races
@@ -237,18 +176,18 @@ class Mount(object):
         """Remove partitions of the device from the file system namespace."""
         if not self.mapped:
             return
-        LOG.debug("Unmap dev %s", self.device)
+        LOG.debug(_("Unmap dev %s"), self.device)
         if self.partition and not self.automapped:
-            nova.privsep.fs.remove_device_maps(self.device)
+            utils.execute('kpartx', '-d', self.device, run_as_root=True)
         self.mapped = False
         self.automapped = False
 
     def mnt_dev(self):
         """Mount the device into the file system."""
-        LOG.debug("Mount %(dev)s on %(dir)s",
+        LOG.debug(_("Mount %(dev)s on %(dir)s") %
                   {'dev': self.mapped_device, 'dir': self.mount_dir})
-        out, err = nova.privsep.fs.mount(None, self.mapped_device,
-                                         self.mount_dir, None)
+        _out, err = utils.trycmd('mount', self.mapped_device, self.mount_dir,
+                                 discard_warnings=True, run_as_root=True)
         if err:
             self.error = _('Failed to mount filesystem: %s') % err
             LOG.debug(self.error)
@@ -261,14 +200,10 @@ class Mount(object):
         """Unmount the device from the file system."""
         if not self.mounted:
             return
-        self.flush_dev()
-        LOG.debug("Umount %s", self.mapped_device)
-        nova.privsep.fs.umount(self.mapped_device)
+        LOG.debug(_("Umount %s") % self.mapped_device)
+        utils.execute('umount', self.mapped_device, run_as_root=True)
         self.mounted = False
 
-    def flush_dev(self):
-        pass
-
     def do_mount(self):
         """Call the get, map and mnt operations."""
         status = False
@@ -276,7 +211,7 @@ class Mount(object):
             status = self.get_dev() and self.map_dev() and self.mnt_dev()
         finally:
             if not status:
-                LOG.debug("Fail to mount, tearing back down")
+                LOG.debug(_("Fail to mount, tearing back down"))
                 self.do_teardown()
         return status
 
