@@ -430,29 +430,19 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    def pre_live_migration(self, ctxt, instance_ref,
-                           block_device_info, network_info,
-                           migrate_data=None):
+    def pre_live_migration(self, ctxt, instance_ref, block_device_info,
+                           network_info, disk_info, migrate_data=None):
         """Prepare an instance for live migration
 
         :param ctxt: security context
         :param instance_ref: instance object that will be migrated
         :param block_device_info: instance block device information
         :param network_info: instance network information
+        :param disk_info: instance disk information
         :param migrate_data: implementation specific data dict.
         """
         raise NotImplementedError()
 
-    def pre_block_migration(self, ctxt, instance_ref, disk_info):
-        """Prepare a block device for migration
-
-        :param ctxt: security context
-        :param instance_ref: instance object that will have its disk migrated
-        :param disk_info: information about disk to be migrated (as returned
-                          from get_instance_disk_info())
-        """
-        raise NotImplementedError()
-
     def live_migration(self, ctxt, instance_ref, dest,
                        post_method, recover_method, block_migration=False,
                        migrate_data=None):
