@@ -70,3 +70,9 @@ VM_SUMMARY_UPTIME = 105
 
 IDE_DISK = "VHD"
 IDE_DVD = "DVD"
+
+DISK_FORMAT_VHD = "VHD"
+DISK_FORMAT_VHDX = "VHDX"
+
+VHD_TYPE_FIXED = 2
+VHD_TYPE_DYNAMIC = 3
