@@ -100,7 +100,7 @@ def set_vif_host_backend_802qbh_config(conf, devname, profileid,
         conf.target_dev = tapname
 
 
-def set_vif_bandwidth_config(conf, extra_specs):
+def set_vif_bandwidth_config(conf, inst_type):
     """Config vif inbound/outbound bandwidth limit. parameters are
     set in instance_type_extra_specs table, key is in  the format
     quota:vif_inbound_average.
@@ -109,7 +109,7 @@ def set_vif_bandwidth_config(conf, extra_specs):
     bandwidth_items = ['vif_inbound_average', 'vif_inbound_peak',
         'vif_inbound_burst', 'vif_outbound_average', 'vif_outbound_peak',
         'vif_outbound_burst']
-    for key, value in extra_specs.iteritems():
+    for key, value in inst_type['extra_specs'].iteritems():
         scope = key.split(':')
         if len(scope) > 1 and scope[0] == 'quota':
             if scope[1] in bandwidth_items:
