@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2012 IBM
+# Copyright 2012 IBM Corp.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -65,6 +65,9 @@ class BaseCommand(object):
     def vhost_by_instance_id(self, instance_id_hex):
         pass
 
+    def chsyscfg(self, args=''):
+        return 'chsyscfg %s' % args
+
 
 class IVMCommand(BaseCommand):
 
