@@ -37,6 +37,7 @@ A connection to the VMware ESX platform.
 :use_linked_clone:          Whether to use linked clone (default: True)
 """
 
+import re
 import time
 
 from eventlet import event
@@ -107,8 +108,17 @@ vmwareapi_opts = [
                 help='Whether to use linked clone'),
     ]
 
+vmware_opts = [
+    cfg.StrOpt('datastore_regex',
+               default=None,
+               help='Regex to match the name of a datastore. '
+                    'Used only if compute_driver is '
+                    'vmwareapi.VMwareVCDriver.'),
+    ]
+
 CONF = cfg.CONF
 CONF.register_opts(vmwareapi_opts)
+CONF.register_opts(vmware_opts, 'vmware')
 
 TIME_BETWEEN_API_CALL_RETRIES = 2.0
 
@@ -359,6 +369,20 @@ class VMwareVCDriver(VMwareESXDriver):
             if self._cluster is None:
                 raise exception.NotFound(_("VMware Cluster %s is not found")
                                            % self._cluster_name)
+
+        self._datastore_regex = None
+        if CONF.vmware.datastore_regex:
+            try:
+                self._datastore_regex = re.compile(CONF.vmware.datastore_regex)
+            except re.error:
+                raise exception.InvalidInput(reason=
+                _("Invalid Regular Expression %s")
+                % CONF.vmware.datastore_regex)
+        self._volumeops = volumeops.VMwareVolumeOps(self._session,
+                                                    self._cluster_name)
+        self._vmops = vmops.VMwareVCVMOps(self._session, self.virtapi,
+                                          self._volumeops, self._cluster_name,
+                                          self._datastore_regex)
         self._vc_state = None
 
     @property
