@@ -37,6 +37,7 @@ from nova import context as nova_context
 from nova import exception
 from nova.openstack.common import excutils
 from nova.openstack.common import log as logging
+from nova import utils
 from nova.virt import driver
 from nova.virt.vmwareapi import vif as vmwarevif
 from nova.virt.vmwareapi import vim_util
@@ -74,7 +75,8 @@ RESIZE_TOTAL_STEPS = 4
 class VMwareVMOps(object):
     """Management class for VM-related tasks."""
 
-    def __init__(self, session, virtapi, volumeops, cluster_name=None):
+    def __init__(self, session, virtapi, volumeops, cluster_name=None,
+                 datastore_regex=None):
         """Initializer."""
         self.compute_api = compute.API()
         self._session = session
@@ -85,10 +87,12 @@ class VMwareVMOps(object):
         else:
             self._cluster = vm_util.get_cluster_ref_from_name(
                                         self._session, cluster_name)
+        self._datastore_regex = datastore_regex
         self._instance_path_base = VMWARE_PREFIX + CONF.base_dir_name
         self._default_root_device = 'vda'
         self._rescue_suffix = '-rescue'
         self._poll_rescue_last_ran = None
+        self._is_quantum = utils.is_quantum()
 
     def list_instances(self):
         """Lists the VM instances that are registered with the ESX host."""
@@ -136,7 +140,8 @@ class VMwareVMOps(object):
 
         client_factory = self._session._get_vim().client.factory
         service_content = self._session._get_vim().get_service_content()
-        ds = vm_util.get_datastore_ref_and_name(self._session, self._cluster)
+        ds = vm_util.get_datastore_ref_and_name(self._session, self._cluster,
+                 datastore_regex=self._datastore_regex)
         data_store_ref = ds[0]
         data_store_name = ds[1]
 
@@ -174,14 +179,10 @@ class VMwareVMOps(object):
                 mac_address = vif['address']
                 network_name = vif['network']['bridge'] or \
                                CONF.vmware.integration_bridge
-                if vif['network'].get_meta('should_create_vlan', False):
-                    network_ref = vmwarevif.ensure_vlan_bridge(
-                                                        self._session, vif,
-                                                        self._cluster)
-                else:
-                    # FlatDHCP network without vlan.
-                    network_ref = vmwarevif.ensure_vlan_bridge(
-                        self._session, vif, self._cluster, create_vlan=False)
+                network_ref = vmwarevif.get_network_ref(self._session,
+                                                        self._cluster,
+                                                        vif,
+                                                        self._is_quantum)
                 vif_infos.append({'network_name': network_name,
                                   'mac_address': mac_address,
                                   'network_ref': network_ref,
@@ -424,6 +425,12 @@ class VMwareVMOps(object):
             LOG.debug(_("Powered on the VM instance"), instance=instance)
         _power_on_vm()
 
+    def _get_copy_virtual_disk_spec(self, client_factory, adapter_type,
+                                    disk_type):
+        return vm_util.get_copy_virtual_disk_spec(client_factory,
+                                                  adapter_type,
+                                                  disk_type)
+
     def snapshot(self, context, instance, snapshot_name, update_task_state):
         """Create snapshot from a running VM instance.
 
@@ -516,9 +523,9 @@ class VMwareVMOps(object):
         def _copy_vmdk_content():
             # Copy the contents of the disk ( or disks, if there were snapshots
             # done earlier) to a temporary vmdk file.
-            copy_spec = vm_util.get_copy_virtual_disk_spec(client_factory,
-                                                           adapter_type,
-                                                           disk_type)
+            copy_spec = self._get_copy_virtual_disk_spec(client_factory,
+                                                         adapter_type,
+                                                         disk_type)
             LOG.debug(_('Copying disk data before snapshot of the VM'),
                       instance=instance)
             copy_disk_task = self._session._call_method(
@@ -1316,3 +1323,22 @@ class VMwareVMOps(object):
     def unplug_vifs(self, instance, network_info):
         """Unplug VIFs from networks."""
         pass
+
+
+class VMwareVCVMOps(VMwareVMOps):
+    """Management class for VM-related tasks.
+
+    Contains specializations to account for differences in vSphere API behavior
+    when invoked on Virtual Center instead of ESX host.
+    """
+
+    def _get_copy_virtual_disk_spec(self, client_factory, adapter_type,
+                                    disk_type):
+        LOG.debug(_("Will copy while retaining adapter type "
+                    "%(adapter_type)s and disk type %(disk_type)s") %
+                    {"disk_type": disk_type,
+                     "adapter_type": adapter_type})
+        # Passing of the destination copy spec is not supported when
+        # VirtualDiskManager.CopyVirtualDisk is called on VC. The behavior of a
+        # spec-less copy is to consolidate to the target disk while keeping its
+        # disk and adapter type unchanged.
