@@ -1,6 +1,7 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright (c) 2010 Citrix Systems, Inc.
+# Copyright (c) 2013 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -22,108 +23,65 @@ and storage repositories
 import re
 import string
 
-from nova import flags
+from oslo.config import cfg
+
 from nova.openstack.common import log as logging
 
-
-FLAGS = flags.FLAGS
+CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
 class StorageError(Exception):
-    """To raise errors related to SR, VDI, PBD, and VBD commands"""
+    """To raise errors related to SR, VDI, PBD, and VBD commands."""
 
     def __init__(self, message=None):
         super(StorageError, self).__init__(message)
 
 
-def create_sr(session, label, params):
-    LOG.debug(_("creating sr within volume_utils"))
-    type = params['sr_type']
-    del params['sr_type']
-    LOG.debug(_('type is = %s') % type)
-    if 'name_description' in params:
-        desc = params['name_description']
-        LOG.debug(_('name = %s') % desc)
-        del params['name_description']
-    else:
-        desc = ''
+def _handle_sr_params(params):
     if 'id' in params:
         del params['id']
-    LOG.debug(params)
 
-    try:
-        sr_ref = session.call_xenapi("SR.create",
-                    session.get_xenapi_host(),
-                    params,
-                    '0', label, desc, type, '', False, {})
-        LOG.debug(_('Created %(label)s as %(sr_ref)s.') % locals())
-        return sr_ref
+    sr_type = params.pop('sr_type', 'iscsi')
+    sr_desc = params.pop('name_description', '')
+    return sr_type, sr_desc
 
-    except session.XenAPI.Failure, exc:
-        LOG.exception(exc)
-        raise StorageError(_('Unable to create Storage Repository'))
+
+def create_sr(session, label, params):
+    LOG.debug(_('Creating SR %(label)s') % locals())
+    sr_type, sr_desc = _handle_sr_params(params)
+    sr_ref = session.call_xenapi("SR.create",
+                session.get_xenapi_host(),
+                params,
+                '0', label, sr_desc, sr_type, '', False, {})
+    return sr_ref
 
 
 def introduce_sr(session, sr_uuid, label, params):
-    LOG.debug(_("introducing sr within volume_utils"))
-    type = params['sr_type']
-    del params['sr_type']
-    LOG.debug(_('type is = %s') % type)
-    if 'name_description' in params:
-        desc = params['name_description']
-        LOG.debug(_('name = %s') % desc)
-        del params['name_description']
-    else:
-        desc = ''
-    if 'id' in params:
-        del params['id']
-    LOG.debug(params)
+    LOG.debug(_('Introducing SR %(label)s') % locals())
 
-    try:
-        sr_ref = session.call_xenapi("SR.introduce",
-                                      sr_uuid,
-                                      label,
-                                      desc,
-                                      type,
-                                      '',
-                                      False,
-                                      params,)
-        LOG.debug(_('Introduced %(label)s as %(sr_ref)s.') % locals())
+    sr_type, sr_desc = _handle_sr_params(params)
 
-        #Create pbd
-        LOG.debug(_('Creating pbd for SR'))
-        pbd_ref = create_pbd(session, sr_ref, params)
-        LOG.debug(_('Plugging SR'))
-        #Plug pbd
-        session.call_xenapi("PBD.plug", pbd_ref)
-        session.call_xenapi("SR.scan", sr_ref)
-        return sr_ref
+    sr_ref = session.call_xenapi('SR.introduce', sr_uuid, label, sr_desc,
+            sr_type, '', False, params)
 
-    except session.XenAPI.Failure, exc:
-        LOG.exception(exc)
-        raise StorageError(_('Unable to introduce Storage Repository'))
+    LOG.debug(_('Creating PBD for SR'))
+    pbd_ref = create_pbd(session, sr_ref, params)
+
+    LOG.debug(_('Plugging SR'))
+    session.call_xenapi("PBD.plug", pbd_ref)
+
+    session.call_xenapi("SR.scan", sr_ref)
+    return sr_ref
 
 
-def forget_sr(session, sr_uuid):
+def forget_sr(session, sr_ref):
     """
     Forgets the storage repository without destroying the VDIs within
     """
-    try:
-        sr_ref = session.call_xenapi("SR.get_by_uuid", sr_uuid)
-    except session.XenAPI.Failure, exc:
-        LOG.exception(exc)
-        raise StorageError(_('Unable to get SR using uuid'))
-
-    LOG.debug(_('Forgetting SR %s...') % sr_ref)
-
-    try:
-        unplug_pbds(session, sr_ref)
-        sr_ref = session.call_xenapi("SR.forget", sr_ref)
-
-    except session.XenAPI.Failure, exc:
-        LOG.exception(exc)
-        raise StorageError(_('Unable to forget Storage Repository'))
+    LOG.debug(_('Forgetting SR...'))
+    unplug_pbds(session, sr_ref)
+    session.call_xenapi("SR.forget", sr_ref)
 
 
 def find_sr_by_uuid(session, sr_uuid):
@@ -136,37 +94,8 @@ def find_sr_by_uuid(session, sr_uuid):
     return None
 
 
-def create_iscsi_storage(session, info, label, description):
-    """
-    Create an iSCSI storage repository that will be used to mount
-    the volume for the specified instance
-    """
-    sr_ref = session.call_xenapi("SR.get_by_name_label", label)
-    if len(sr_ref) == 0:
-        LOG.debug(_('Introducing %s...'), label)
-        record = {}
-        if 'chapuser' in info and 'chappassword' in info:
-            record = {'target': info['targetHost'],
-                      'port': info['targetPort'],
-                      'targetIQN': info['targetIQN'],
-                      'chapuser': info['chapuser'],
-                      'chappassword': info['chappassword']}
-        else:
-            record = {'target': info['targetHost'],
-                      'port': info['targetPort'],
-                      'targetIQN': info['targetIQN']}
-        try:
-            LOG.debug(_('Introduced %(label)s as %(sr_ref)s.') % locals())
-            return sr_ref
-        except session.XenAPI.Failure, exc:
-            LOG.exception(exc)
-            raise StorageError(_('Unable to create Storage Repository'))
-    else:
-        return sr_ref[0]
-
-
 def find_sr_from_vbd(session, vbd_ref):
-    """Find the SR reference from the VBD reference"""
+    """Find the SR reference from the VBD reference."""
     try:
         vdi_ref = session.call_xenapi("VBD.get_VDI", vbd_ref)
         sr_ref = session.call_xenapi("VDI.get_SR", vdi_ref)
@@ -186,22 +115,23 @@ def create_pbd(session, sr_ref, params):
 
 
 def unplug_pbds(session, sr_ref):
-    pbds = []
     try:
         pbds = session.call_xenapi("SR.get_PBDs", sr_ref)
     except session.XenAPI.Failure, exc:
         LOG.warn(_('Ignoring exception %(exc)s when getting PBDs'
-                ' for %(sr_ref)s') % locals())
+                   ' for %(sr_ref)s') % locals())
+        return
+
     for pbd in pbds:
         try:
             session.call_xenapi("PBD.unplug", pbd)
         except session.XenAPI.Failure, exc:
             LOG.warn(_('Ignoring exception %(exc)s when unplugging'
-                    ' PBD %(pbd)s') % locals())
+                       ' PBD %(pbd)s') % locals())
 
 
 def introduce_vdi(session, sr_ref, vdi_uuid=None, target_lun=None):
-    """Introduce VDI in the host"""
+    """Introduce VDI in the host."""
     try:
         session.call_xenapi("SR.scan", sr_ref)
         if vdi_uuid:
@@ -255,69 +185,76 @@ def introduce_vdi(session, sr_ref, vdi_uuid=None, target_lun=None):
 
 
 def purge_sr(session, sr_ref):
-    try:
-        sr_rec = session.call_xenapi("SR.get_record", sr_ref)
-        vdi_refs = session.call_xenapi("SR.get_VDIs", sr_ref)
-    except StorageError, ex:
-        LOG.exception(ex)
-        raise StorageError(_('Error finding vdis in SR %s') % sr_ref)
-
+    # Make sure no VBDs are referencing the SR VDIs
+    vdi_refs = session.call_xenapi("SR.get_VDIs", sr_ref)
     for vdi_ref in vdi_refs:
-        try:
-            vbd_refs = session.call_xenapi("VDI.get_VBDs", vdi_ref)
-        except StorageError, ex:
-            LOG.exception(ex)
-            raise StorageError(_('Unable to find vbd for vdi %s') %
-                               vdi_ref)
-        if len(vbd_refs) > 0:
+        vbd_refs = session.call_xenapi("VDI.get_VBDs", vdi_ref)
+        if vbd_refs:
+            LOG.warn(_('Cannot purge SR with referenced VDIs'))
             return
 
-    forget_sr(session, sr_rec['uuid'])
+    forget_sr(session, sr_ref)
 
 
-def parse_volume_info(connection_info, mountpoint):
+def get_device_number(mountpoint):
+    device_number = mountpoint_to_number(mountpoint)
+    if device_number < 0:
+        raise StorageError(_('Unable to obtain target information'
+                ' %(mountpoint)s') % locals())
+    return device_number
+
+
+def parse_sr_info(connection_data, description=''):
+    label = connection_data.pop('name_label',
+                                'tempSR-%s' % connection_data.get('volume_id'))
+    params = {}
+    if 'sr_uuid' not in connection_data:
+        params = parse_volume_info(connection_data)
+        # This magic label sounds a lot like 'False Disc' in leet-speak
+        uuid = "FA15E-D15C-" + str(params['id'])
+    else:
+        uuid = connection_data['sr_uuid']
+        for k in connection_data.get('introduce_sr_keys', {}):
+            params[k] = connection_data[k]
+    params['name_description'] = connection_data.get('name_description',
+                                                     description)
+
+    return (uuid, label, params)
+
+
+def parse_volume_info(connection_data):
     """
     Parse device_path and mountpoint as they can be used by XenAPI.
     In particular, the mountpoint (e.g. /dev/sdc) must be translated
     into a numeric literal.
-    FIXME(armando):
-    As for device_path, currently cannot be used as it is,
-    because it does not contain target information. As for interim
-    solution, target details are passed either via Flags or obtained
-    by iscsiadm. Long-term solution is to add a few more fields to the
-    db in the iscsi_target table with the necessary info and modify
-    the iscsi driver to set them.
     """
-    device_number = mountpoint_to_number(mountpoint)
-    data = connection_info['data']
-    volume_id = data['volume_id']
-    target_portal = data['target_portal']
+    volume_id = connection_data['volume_id']
+    target_portal = connection_data['target_portal']
     target_host = _get_target_host(target_portal)
     target_port = _get_target_port(target_portal)
-    target_iqn = data['target_iqn']
+    target_iqn = connection_data['target_iqn']
     LOG.debug('(vol_id,number,host,port,iqn): (%s,%s,%s,%s)',
               volume_id, target_host, target_port, target_iqn)
-    if (device_number < 0 or
-        volume_id is None or
+    if (volume_id is None or
         target_host is None or
         target_iqn is None):
         raise StorageError(_('Unable to obtain target information'
-                ' %(data)s, %(mountpoint)s') % locals())
+                ' %(connection_data)s') % locals())
     volume_info = {}
     volume_info['id'] = volume_id
     volume_info['target'] = target_host
     volume_info['port'] = target_port
     volume_info['targetIQN'] = target_iqn
-    if ('auth_method' in connection_info and
-        connection_info['auth_method'] == 'CHAP'):
-        volume_info['chapuser'] = connection_info['auth_username']
-        volume_info['chappassword'] = connection_info['auth_password']
+    if ('auth_method' in connection_data and
+        connection_data['auth_method'] == 'CHAP'):
+        volume_info['chapuser'] = connection_data['auth_username']
+        volume_info['chappassword'] = connection_data['auth_password']
 
     return volume_info
 
 
 def mountpoint_to_number(mountpoint):
-    """Translate a mountpoint like /dev/sdc into a numeric"""
+    """Translate a mountpoint like /dev/sdc into a numeric."""
     if mountpoint.startswith('/dev/'):
         mountpoint = mountpoint[5:]
     if re.match('^[hs]d[a-p]$', mountpoint):
@@ -332,7 +269,7 @@ def mountpoint_to_number(mountpoint):
 
 
 def _get_volume_id(path_or_id):
-    """Retrieve the volume id from device_path"""
+    """Retrieve the volume id from device_path."""
     # If we have the ID and not a path, just return it.
     if isinstance(path_or_id, int):
         return path_or_id
@@ -351,25 +288,16 @@ def _get_volume_id(path_or_id):
 
 
 def _get_target_host(iscsi_string):
-    """Retrieve target host"""
+    """Retrieve target host."""
     if iscsi_string:
         return iscsi_string[0:iscsi_string.find(':')]
-    elif iscsi_string is None or FLAGS.target_host:
-        return FLAGS.target_host
+    elif iscsi_string is None or CONF.target_host:
+        return CONF.target_host
 
 
 def _get_target_port(iscsi_string):
-    """Retrieve target port"""
+    """Retrieve target port."""
     if iscsi_string:
         return iscsi_string[iscsi_string.find(':') + 1:]
-    elif iscsi_string is None or FLAGS.target_port:
-        return FLAGS.target_port
-
-
-def _get_iqn(iscsi_string, id):
-    """Retrieve target IQN"""
-    if iscsi_string:
-        return iscsi_string
-    elif iscsi_string is None or FLAGS.iqn_prefix:
-        volume_id = _get_volume_id(id)
-        return '%s:%s' % (FLAGS.iqn_prefix, volume_id)
+    elif iscsi_string is None or CONF.target_port:
+        return CONF.target_port
